﻿using FLY.FeedbackRenZiJia.Common;
using FLY.FeedbackRenZiJia.IService;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;

namespace FLY.FeedbackRenZiJia.UI.Client
{
    public class HeatListHelper : INotifyPropertyChanged
    {
        const int MARKNO_UPDATED = 1;
        const int MARKNO_SHOWD = 2;
        #region 参数

        /// <summary>
        /// 比例因子
        /// </summary>
        public double Factor { get; set; } = 3;



        /// <summary>
        /// 复位区号
        /// </summary>
        public int OrgBoltNo { get; set; } = 1;



        /// <summary>
        /// 旋转角度
        /// </summary>
        public double RAngle { get; set; } = 350;



        /// <summary>
        /// 反向
        /// </summary>
        public bool IsReversed { get; set; } = false;

        #endregion
        #region 状态
        private double currmaxmin = -1;
        /// <summary>
        /// 当前厚度%变化极差 单位%
        /// </summary>
        public double CurrMaxMin
        {
            get { return currmaxmin; }
            set
            {
                if (currmaxmin != value)
                {
                    currmaxmin = value;
                    NotifyPropertyChanged("CurrMaxMin");
                }
            }
        }
        private double currHeatThickR = -1;
        /// <summary>
        /// 当前加热与厚度相关性
        /// </summary>
        public double CurrHeatThickR
        {
            get { return currHeatThickR; }
            set
            {
                if (currHeatThickR != value)
                {
                    currHeatThickR = value;
                    NotifyPropertyChanged("CurrHeatThickR");
                }
            }
        }

        private double currFactor = -1;
        /// <summary>
        /// 当前加热与厚度 比例
        /// </summary>
        public double CurrFactor
        {
            get { return currFactor; }
            set
            {
                if (currFactor != value)
                {
                    currFactor = value;
                    NotifyPropertyChanged("CurrFactor");
                }
            }
        }
        #endregion


        int NBolts;
        int ChannelCnt;

        int Org_OrgBoltNo;
        double Org_RAngle;
        int[] Org_ThickPercentDiff;
        int[] Org_HeatDiff;

        /// <summary>
        /// 数量为NBolts
        /// </summary>
        public int[] ThickPercentDiff;
        public int[] HeatDiff;

        FLY.FeedbackRenZiJia.Client.FeedbackHeatServiceClient mFeedback;
        FLY.FeedbackRenZiJia.Client.HeatCellServiceClient mHeatCell;

        public HeatListHelper()
        {

        }


        public void Init(SnapShotList snaps)
        {
            mFeedback = TDGage.Current.mFeedback;
            mHeatCell = TDGage.Current.mHeatCell;

            mFeedback.PropertyChanged += FeedbackHeat_PropertyChanged;
            
            this.PropertyChanged += HeatListHelper_PropertyChanged;

            Clear(mFeedback.NBolts, mFeedback.ChannelCnt);

            Misc.BindingOperations.SetBinding(mHeatCell, "Kp", this, "Factor");

            //Misc.BindingOperations.SetBinding(mBlowing, "OrgBoltNo", this, "OrgBoltNo");
            
            //Misc.BindingOperations.SetBinding(mBDetect, "RAngle", this, "RAngle");

            snaps.PropertyChanged += Snaps_PropertyChanged;
        }

        private void Snaps_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "CurrData")
            {
                SnapShotList snaps = sender as SnapShotList;
                if (snaps.CurrData != null)
                {
                    UpdateD(
                        snaps.CurrData.Thicks,
                        snaps.CurrData.Heats,
                        snaps.CurrData.RAngle,
                        snaps.CurrData.OrgBoltNo);
                }
            }
        }

        private void HeatListHelper_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if ((e.PropertyName == "RAngle") ||
                (e.PropertyName == "OrgBoltNo") ||
                (e.PropertyName == "IsReversed"))
            {
                FObjBase.PollModule.Current.Poll_JustOnce(
                    () =>
                    {
                        UpdateD();
                    }, this, MARKNO_UPDATED);
            }
        }

        void UpdateD()
        {

            int[] org_diff = Org_ThickPercentDiff;
            int[] heat_diff = Org_HeatDiff;
            double rangle = Org_RAngle;
            int orgboltno = Org_OrgBoltNo;
            if (org_diff == null)
                return;

            //反向
            if (IsReversed)
            {
                org_diff = FLY.FeedbackRenZiJia.Common.MyMath.ReverseX(org_diff, orgboltno - 1);
            }

            //放大
            int[] diff = FLY.FeedbackRenZiJia.Common.MyMath.ZoomX(org_diff, orgboltno - 1, RAngle / rangle);
            //平移
            diff = FLY.FeedbackRenZiJia.Common.MyMath.Move(diff, OrgBoltNo - orgboltno);
            Array.Copy(diff, ThickPercentDiff, NBolts);
            Array.Copy(heat_diff, HeatDiff, ChannelCnt);

            FObjBase.PollModule.Current.Poll_JustOnce(
                () =>
                {
                    NotifyPropertyChanged("DataDiff");
                }, this, MARKNO_SHOWD);
            int bpc = NBolts / ChannelCnt;
            //缩小数据量
            diff = FLY.FeedbackRenZiJia.Common.MyMath.ZoomOut(ThickPercentDiff, bpc);

            //-----------------------------------------------------------------------------------------------------------------------------------------
            //厚度%差 极差，判断可信度用
            int min = Misc.MyMath.Min(diff);
            int max = Misc.MyMath.Max(diff);
            CurrMaxMin = max - min;

            //-----------------------------------------------------------------------------------------------------------------------------------------
            //当前数据的加热与厚度相关性
            CurrHeatThickR = Misc.MyMath.Correl(diff, heat_diff);

            //-----------------------------------------------------------------------------------------------------------------------------------------
            //当前数据的加热与厚度 比例
            double b;
            double a;
            Misc.MyMath.Linest(heat_diff, diff, out a, out b);
            CurrFactor = a;
        }

        /// <summary>
        /// 更新两个图的偏差 
        /// </summary>
        void UpdateD(int[] org_diff, int[] heat_diff, double rangle, int orgboltno)
        {
            Org_ThickPercentDiff = org_diff;
            Org_HeatDiff = heat_diff;
            Org_RAngle = rangle;
            Org_OrgBoltNo = orgboltno;
            UpdateD();
        }

        /// <summary>
        /// 删除全部数据！！！！
        /// </summary>
        /// <param name="nbolts"></param>
        /// <param name="channelcnt"></param>
        void Clear(int nbolts, int channelcnt)
        {
            ChannelCnt = channelcnt;
            NBolts = nbolts;

            //经过放大移动后的数据
            ThickPercentDiff = new int[NBolts];
            HeatDiff = new int[ChannelCnt];

            for (int i = 0; i < NBolts; i++)
            {
                ThickPercentDiff[i] = Misc.MyBase.NULL_VALUE;
            }
            NotifyPropertyChanged("DataDiff");
        }

        private void FeedbackHeat_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if ((e.PropertyName == "ChannelCnt") || (e.PropertyName == "NBolts"))
            {
                Clear(mFeedback.NBolts, mFeedback.ChannelCnt);
            }
        }

        public void Auto()
        {
            double bestr = -1;
            int bestOrgBoltNo = OrgBoltNo;
            for (int i = 0; i < NBolts; i++)
            {
                if (OrgBoltNo < NBolts)
                    OrgBoltNo++;
                else
                    OrgBoltNo = 1;

                UpdateD();
                if (CurrHeatThickR > bestr)
                {
                    bestr = CurrHeatThickR;
                    bestOrgBoltNo = OrgBoltNo;
                }
            }
            OrgBoltNo = bestOrgBoltNo;

            double rAngle_min = 300;
            double rAngle_max = 358;
            double bestRAngle = RAngle;
            for (double i = rAngle_min; i < rAngle_max; i++)
            {
                RAngle = i;
                UpdateD();
                if (CurrHeatThickR > bestr)
                {
                    bestr = CurrHeatThickR;
                    bestRAngle = RAngle;
                }
            }
            RAngle = bestRAngle;

            UpdateD();
            Factor = CurrFactor;

        }

        #region INotifyPropertyChanged
        protected void NotifyPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion
    }
    public enum HeatListCalResult
    {
        OK,
        /// <summary>
        /// 无数据
        /// </summary>
        Error_NoData,
        /// <summary>
        /// 复位区号不相同
        /// </summary>
        Error_DiffWithOrgBoltNo,
        /// <summary>
        /// 旋转角度不相同
        /// </summary>
        Error_DiffWithRAngle
    }
    public class FlyData_FeedbackHeat2 : INotifyPropertyChanged
    {
        private bool isvalid = false;
        /// <summary>
        /// 有效
        /// </summary>
        public bool IsVaild
        {
            get
            {
                return isvalid;
            }
            set
            {
                if (isvalid != value)
                {
                    isvalid = value;
                    NotifyPropertyChanged("IsVaild");
                }
            }
        }

        private bool isTakedEffect = true;
        /// <summary>
        /// 加热起效
        /// </summary>
        public bool IsTakedEffect
        {
            get
            {
                return isTakedEffect;
            }
            set
            {
                if (isTakedEffect != value)
                {
                    isTakedEffect = value;
                    NotifyPropertyChanged("IsTakedEffect");
                }
            }
        }

        private DateTime time;
        /// <summary>
        /// 开始测量时间！！！
        /// </summary>
        public DateTime Time
        {
            get {
                return time;
            }
            set {
                if (time != value)
                {
                    time = value;
                    NotifyPropertyChanged("Time");
                }
            }
        }

        private DateTime endtime;
        /// <summary>
        /// 结束测量时间！！！
        /// </summary>
        public DateTime EndTime
        {
            get
            {
                return endtime;
            }
            set
            {
                if (endtime != value)
                {
                    endtime = value;
                    NotifyPropertyChanged("EndTime");
                }
            }
        }

        private Misc.DIRECTION direction;
        /// <summary>
        /// 方向
        /// </summary>
        public Misc.DIRECTION Direction
        {
            get
            {
                return direction;
            }
            set
            {
                if (direction != value)
                {
                    direction = value;
                    NotifyPropertyChanged("Direction");
                }
            }
        }

        private int orgboltno;
        /// <summary>
        /// 复位区号
        /// </summary>
        public int OrgBoltNo
        {
            get { return orgboltno; }
            set {
                if (orgboltno != value)
                {
                    orgboltno = value;
                    NotifyPropertyChanged("OrgBoltNo");
                }
            }
        }

        private double rangle;
        /// <summary>
        /// 旋转架旋转角度
        /// </summary>
        public double RAngle
        {
            get { return rangle; }
            set
            {
                if (rangle != value)
                {
                    rangle = value;
                    NotifyPropertyChanged("RAngle");
                }
            }
        }

        private double filmlength;
        /// <summary>
        /// 膜距离
        /// </summary>
        public double FilmLength
        {
            get { return filmlength; }
            set
            {
                if (filmlength != value)
                {
                    filmlength = value;
                    NotifyPropertyChanged("FilmLength");
                }
            }
        }

        private TimeSpan rotateperiod;
        /// <summary>
        /// 旋转时间
        /// </summary>
        public TimeSpan RotatePeriod
        {
            get
            {
                return rotateperiod;
            }
            set
            {
                if (rotateperiod != value)
                {
                    rotateperiod = value;
                    NotifyPropertyChanged("RotatePeriod");
                }
            }
        }

        private DateTime htime;
        /// <summary>
        /// 开始改变加热时间
        /// </summary>
        public DateTime HTime
        {
            get
            {
                return htime;
            }
            set
            {
                if (htime != value)
                {
                    htime = value;
                    NotifyPropertyChanged("HTime");
                }
            }
        }

        #region INotifyPropertyChanged
        protected void NotifyPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion
    }


    /// <summary>
    /// 向列表读取2幅数据，并得到差值
    /// </summary>
    public class HeatList2 : INotifyPropertyChanged
    {
        const int MARKNO_UPDATED = 1;

        #region 参数
        private int bookmark1 = 0;
        /// <summary>
        /// 图1记录点
        /// </summary>
        public int Bookmark1
        {
            get { return bookmark1; }
            set
            {
                if (bookmark1 != value)
                {
                    bookmark1 = value;
                    NotifyPropertyChanged("Bookmark1");
                }
            }
        }

        private int bookmark2 = 0;
        /// <summary>
        /// 图2记录点
        /// </summary>
        public int Bookmark2
        {
            get { return bookmark2; }
            set
            {
                if (bookmark2 != value)
                {
                    bookmark2 = value;
                    NotifyPropertyChanged("Bookmark2");
                }
            }
        }

        private HeatListCalResult calResult = HeatListCalResult.OK;
        //差分计算结果
        public HeatListCalResult CalResult
        {
            get
            {
                return calResult;
            }
            protected set
            {
                if (calResult != value)
                {
                    calResult = value;
                    NotifyPropertyChanged("CalResult");
                }
            }
        }
        #endregion

        private FlyData_FeedbackHeat _flydata1;
        private FlyData_FeedbackHeat _flydata2;

        private int NBolts = 88;
        /// <summary>
        /// 数量为NBolts
        /// </summary>
        public int[][] ThickPercents = new int[2][];

        private int ChannelCnt = 44;
        /// <summary>
        /// 数量为ChannelCnt
        /// </summary>
        public int[][] Heats = new int[2][];

        /// <summary>
        /// 数量为NBolts
        /// </summary>
        public int[] Org_ThickPercentDiff;

        /// <summary>
        /// 数量为ChannelCnt
        /// </summary>
        public int[] Org_HeatDiff;

        public FlyData_FeedbackHeat2 flydata1 = new FlyData_FeedbackHeat2();
        public FlyData_FeedbackHeat2 flydata2 = new FlyData_FeedbackHeat2();

        public FLY.FeedbackRenZiJia.Client.HeatBufServiceClient mHeatBuf;
        public FLY.FeedbackRenZiJia.Client.FeedbackHeatServiceClient mFeedback;
        public void Init()
        {
            mFeedback = TDGage.Current.mFeedback;
            mHeatBuf = TDGage.Current.mHeatBuf;

            mHeatBuf.PropertyChanged += (s, e) =>
            {
                if ((e.PropertyName == "ChannelCnt") || (e.PropertyName == "NBolts"))
                {
                    Clear(mFeedback.NBolts, mFeedback.ChannelCnt);
                }
            };

            this.PropertyChanged += HeatListHelper_PropertyChanged;

            Clear(mFeedback.NBolts, mFeedback.ChannelCnt);
        }


        bool IsValidBM(int bookmark)
        {
            if (mHeatBuf.FirstBM < mHeatBuf.LastBM)
            {
                if ((bookmark >= mHeatBuf.FirstBM) && (bookmark <= mHeatBuf.LastBM))
                    return true;
                else
                    return false;
            }
            else
            {
                if ((bookmark >= mHeatBuf.FirstBM) || (bookmark <= mHeatBuf.LastBM))
                    return true;
                else
                    return false;
            }
        }

        private void HeatListHelper_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Bookmark1")
            {
                if (IsValidBM(Bookmark1))
                {
                    mHeatBuf.GetHeatsData(Bookmark1,
                        (object s, object retdata) =>
                        {
                            FLY.FeedbackRenZiJia.Common.FlyData_FeedbackHeat p = retdata as FLY.FeedbackRenZiJia.Common.FlyData_FeedbackHeat;
                            _flydata1 = p;

                            UpdateOrg(_flydata1, flydata1, ThickPercents[0], Heats[0]);
                            NotifyPropertyChanged("Data1");
                            UpdateD_Notify();
                        }, this);
                }
            }
            else if (e.PropertyName == "Bookmark2")
            {
                if (IsValidBM(Bookmark2))
                {
                    mHeatBuf.GetHeatsData(Bookmark2,
                        (object s, object retdata) =>
                        {
                            FLY.FeedbackRenZiJia.Common.FlyData_FeedbackHeat p = retdata as FLY.FeedbackRenZiJia.Common.FlyData_FeedbackHeat;
                            _flydata2 = p;
                            UpdateOrg(_flydata2, flydata2, ThickPercents[1], Heats[1]);
                            NotifyPropertyChanged("Data2");
                            UpdateD_Notify();
                        }, this);
                }
            }
        }

        void UpdateD_Notify()
        {
            if (UpdateD())
                NotifyPropertyChanged("DataDiff");
        }

        bool UpdateD()
        {
            //复位区号，旋转角度，必须都一样！！！
            if (flydata1 == null || flydata2 == null)
            {
                CalResult = HeatListCalResult.Error_NoData;
                return false;
            }

            if (flydata1.OrgBoltNo != flydata2.OrgBoltNo)
            {
                CalResult = HeatListCalResult.Error_DiffWithOrgBoltNo;
                return false;
            }

            if (flydata1.RAngle != flydata2.RAngle)
            {
                CalResult = HeatListCalResult.Error_DiffWithRAngle;
                return false;
            }


            for (int i = 0; i < NBolts; i++)
            {
                if ((Misc.MyBase.ISVALIDATA(ThickPercents[1][i]) && Misc.MyBase.ISVALIDATA(ThickPercents[0][i])))
                    Org_ThickPercentDiff[i] = ThickPercents[0][i] - ThickPercents[1][i];
                else
                    Org_ThickPercentDiff[i] = Misc.MyBase.NULL_VALUE;
            }

            for (int i = 0; i < ChannelCnt; i++)
            {
                Org_HeatDiff[i] = Heats[1][i] - Heats[0][i];
            }
            
            return true;
        }
        

        /// <summary>
        /// 给定 加热记录， 更新 厚度% 与 加热%
        /// </summary>
        /// <param name="flydata"></param>
        /// <param name="target_flydata"></param>
        /// <param name="thickpercents"></param>
        /// <param name="heats"></param>
        void UpdateOrg(
            FLY.FeedbackRenZiJia.Common.FlyData_FeedbackHeat flydata,
            FlyData_FeedbackHeat2 target_flydata, int[] thickpercents, int[] heats)
        {
            if (flydata == null)
            {
                target_flydata.IsVaild = false;
                return;
            }


            int avg = Misc.MyMath.Avg(flydata.Thicks);
            for (int i = 0; i < NBolts; i++)
            {
                int t = flydata.Thicks[i];
                if (Misc.MyBase.ISVALIDATA(t) && Misc.MyBase.ISVALIDATA(avg) && avg > 0)
                {
                    thickpercents[i] = 100 * (t - avg) / avg;
                }
                else
                {
                    thickpercents[i] = Misc.MyBase.NULL_VALUE;
                }
            }

            Array.Copy(flydata.Heats, heats, Math.Min(flydata.Heats.Count(), ChannelCnt));


            target_flydata.IsVaild = true;
            target_flydata.Time = flydata.Time;
            target_flydata.EndTime = flydata.EndTime;
            target_flydata.Direction = flydata.Direction;
            target_flydata.HTime = flydata.HTime;
            target_flydata.RotatePeriod = flydata.RotatePeriod;
            target_flydata.RAngle = flydata.RAngle;
            target_flydata.OrgBoltNo = flydata.OrgBoltNo;
            target_flydata.FilmLength = flydata.FilmLength;

            DateTime e_dt = target_flydata.HTime + mHeatBuf.Delay;
            if (e_dt <= target_flydata.Time)
            {
                target_flydata.IsTakedEffect = true;
            }
            else if (target_flydata.EndTime == target_flydata.Time)
            {
                target_flydata.IsTakedEffect = false;
            }
            else
            {
                double p = (e_dt - target_flydata.Time).TotalSeconds / (target_flydata.EndTime - target_flydata.Time).TotalSeconds;
                if (p < 0.2)
                    target_flydata.IsTakedEffect = true;
                else
                    target_flydata.IsTakedEffect = false;
            }
        }

        /// <summary>
        /// 删除全部数据！！！！
        /// </summary>
        /// <param name="nbolts"></param>
        /// <param name="channelcnt"></param>
        void Clear(int nbolts, int channelcnt)
        {
            ChannelCnt = channelcnt;
            NBolts = nbolts;
            for (int i = 0; i < 2; i++)
            {
                ThickPercents[i] = new int[NBolts];
                Heats[i] = new int[ChannelCnt];

                for (int j = 0; j < NBolts; j++)
                    ThickPercents[i][j] = Misc.MyBase.NULL_VALUE;
            }
            //两个相减的数据
            Org_ThickPercentDiff = new int[NBolts];
            Org_HeatDiff = new int[ChannelCnt];

            for (int i = 0; i < NBolts; i++)
            {
                Org_ThickPercentDiff[i] = Misc.MyBase.NULL_VALUE;
            }
            NotifyPropertyChanged("Data1");
            NotifyPropertyChanged("Data2");
            NotifyPropertyChanged("DataDiff");
        }

        public void UpdateBookmark()
        {
            Bookmark2 = mHeatBuf.LastBM;
            Bookmark1 = Bookmark2 - 1;
        }

        #region INotifyPropertyChanged
        protected void NotifyPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
        #endregion
    }
    public class SnapShot : INotifyPropertyChanged
    {
        private int bookmark;
        /// <summary>
        /// 记录点
        /// </summary>
        public int Bookmark
        {
            get
            {
                return bookmark;
            }
            set
            {
                if (bookmark != value)
                {
                    bookmark = value;
                    NotifyPropertyChanged("Bookmark");
                }
            }

        }



        private bool isFocusd = false;
        /// <summary>
        /// 获得焦点中
        /// </summary>
        public bool IsFocused
        {
            get
            {
                return isFocusd;
            }
            set
            {
                if (isFocusd != value)
                {
                    isFocusd = value;
                    NotifyPropertyChanged("IsFocused");
                }
            }
        }

        protected void NotifyPropertyChanged(string propertyname)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyname));
        }
        public event PropertyChangedEventHandler PropertyChanged;
    }
    public class SnapShotList : INotifyPropertyChanged
    {
        FLY.FeedbackRenZiJia.Client.SnapShotBufServiceClient mSnapShotBuf;
        HeatList2 h2;

        public ObservableCollection<SnapShot> mList = new ObservableCollection<SnapShot>();//第1个肯定是当前

        private FlyData_SnapShot h2Diff;

        private FlyData_SnapShot currData;
        /// <summary>
        /// 当前的snapshot 数据
        /// </summary>
        public FlyData_SnapShot CurrData
        {
            get { return currData; }
            set
            {
                //if (currData != value)
                {
                    currData = value;
                    //NotifyPropertyChanged("CurrData");
                }
            }
        }

        private int curridx;
        /// <summary>
        /// 当前显示的序号
        /// </summary>
        public int CurrIdx
        {
            get
            {
                return curridx;
            }
            set
            {
                if (curridx != value)
                {
                    curridx = value;
                    NotifyPropertyChanged("CurrIdx");
                }
            }

        }

        private bool isIdx0 = false;

        /// <summary>
        /// 是当前数据
        /// </summary>
        public bool IsIdx0
        {
            get
            {
                return isIdx0;
            }
            set
            {
                if (isIdx0 != value)
                {
                    isIdx0 = value;
                    NotifyPropertyChanged("IsIdx0");
                }
            }
        }



        public SnapShotList()
        {

        }

        public void Init(
            HeatList2 h2
            )
        {
            mSnapShotBuf = TDGage.Current.mSnapShotBuf;
            this.h2 = h2;
            this.h2.PropertyChanged += h2_PropertyChanged;
            //mFeedbackHeat.PropertyChanged += FeedbackHeat_PropertyChanged;

            //this.PropertyChanged += HeatListHelper_PropertyChanged;

            //Clear(mFeedbackHeat.NBolts, mFeedbackHeat.ChannelCnt);

            //Refresh();
        }

        private void h2_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "DataDiff")
            {
                //更新当前的！！！
                if (h2Diff == null)
                    h2Diff = new FlyData_SnapShot();
                    
                h2Diff.BM1_BeginTime = h2.flydata1.Time;
                h2Diff.BM1_EndTime = h2.flydata1.EndTime;
                h2Diff.BM1_Direction = h2.flydata1.Direction;

                h2Diff.BM2_BeginTime = h2.flydata2.Time;
                h2Diff.BM2_EndTime = h2.flydata2.EndTime;
                h2Diff.BM2_Direction = h2.flydata2.Direction;

                h2Diff.OrgBoltNo = h2.flydata1.OrgBoltNo;
                h2Diff.RAngle = h2.flydata1.RAngle;

                h2Diff.Thicks = h2.Org_ThickPercentDiff;
                h2Diff.Heats = h2.Org_HeatDiff;

                if (CurrIdx == 0)
                {
                    CurrData = h2Diff;
                    NotifyPropertyChanged("CurrData");
                }
            }
        }

        public void Refresh()
        {
            mSnapShotBuf.GetList(
                (object AsyncState, object retData) =>
                {
                    List<int> l = retData as List<int>;

                    mList.Clear();
                    mList.Add(new SnapShot());
                    if (l != null)
                    {
                        for (int i = 0; i < l.Count(); i++)
                        {
                            mList.Add(new SnapShot() { Bookmark = l[i] });
                        }
                    }
                    GetSnapShot_Idx(CurrIdx);
                }, this);
        }

        public void GetSnapShot(int bookmark)
        {

            for (int i = 0; i < mList.Count(); i++)
            {
                if (mList[i].Bookmark == bookmark)
                {
                    GetSnapShot_Idx(i);
                }
            }
            //异常
            return;
        }
        void UpdateIsFocused(int idx)
        {
            if (idx >= mList.Count())
                idx = 0;
            else if (idx < 0)
                idx = 0;
            if (CurrIdx >= 0 && CurrIdx < mList.Count())
                mList[CurrIdx].IsFocused = false;

            CurrIdx = idx;
            if (CurrIdx == 0)
                IsIdx0 = true;
            else
                IsIdx0 = false;
            mList[CurrIdx].IsFocused = true;
        }
        void GetSnapShot_Idx(int idx)
        {
            UpdateIsFocused(idx);

            if (CurrIdx != 0)
            {
                mSnapShotBuf.Get(mList[CurrIdx].Bookmark, (object AsyncState, object retData) =>
                {
                    CurrData = retData as FlyData_SnapShot;
                    NotifyPropertyChanged("CurrData");
                }, this);
            }
            else
            {
                CurrData = h2Diff;
                NotifyPropertyChanged("CurrData");
            }
        }

        public void DelSnapShot()
        {
            if (CurrIdx != 0)
            {
                mSnapShotBuf.Del(mList[CurrIdx].Bookmark);
                mList.RemoveAt(CurrIdx);
                GetSnapShot_Idx(CurrIdx);
            }
        }
        public void SetSnapShot()
        {
            if (CurrData.Bookmark == 0)
            {
                Random r = new Random();
                int bookmark;
                while (true)
                {
                    bookmark = r.Next(1000);
                    int cnt = (from s in mList where s.Bookmark == bookmark select s).Count();
                    if (cnt > 0)
                        continue;
                    break;
                }

                FlyData_SnapShot ss = new FlyData_SnapShot();
                ss.Copy(CurrData);
                CurrData = ss;
                ss.Bookmark = bookmark;
                mList.Add(new SnapShot() { Bookmark = bookmark });
                UpdateIsFocused(mList.Count() - 1);
            }
            mSnapShotBuf.Set(CurrData);
            //更新List
        }

        protected void NotifyPropertyChanged(string propertyname)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyname));
        }
        public event PropertyChangedEventHandler PropertyChanged;
    }
}
