﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;
using FLY.FeedbackRenZiJia.OBJ_INTERFACE;
using FObjBase;
using System.Collections.ObjectModel;
using FLY.FeedbackRenZiJia.Common;
using FLY.FeedbackRenZiJia.IService;
using System.Net;

namespace FLY.FeedbackRenZiJia.Client
{
    public class FeedbackHeatServiceClient:FObj,IFeedbackHeat
    {
        IFConn mConn;
        UInt32 mServerID;
        /// <summary>
        /// 连接到服务器
        /// </summary>
        public bool IsConnected { get; set; }

        #region IFeedbackHeat 成员
        #region 参数
        /// <summary>
        /// 作为客户端模式
        /// </summary>
        public bool IsClient { get; set; }
        public IPEndPoint PLCep { get; set; }

        /// <summary>
        /// 自动控制使能
        /// </summary>
        public bool IsAuto { get; set; }

        /// <summary>
        /// 加热后起效延时(s)
        /// </summary>
        public int Delay { get; set; } = 10;

        /// <summary>
        /// 检测功能使能
        /// </summary>
        public bool HasCheck { get; set; }

        /// <summary>
        /// 检测线速度 使能
        /// </summary>
        public bool HasCheckFilmVelocity { get; set; }

        /// <summary>
        /// 手动加热步进
        /// </summary>
        public int Step { get; set; } = 5;


        #endregion


        #region 动态数据
        private int channelcnt = 44;
        /// <summary>
        /// 通道数， 从测厚仪获取的！！！！
        /// </summary>
        public int ChannelCnt { get; protected set; } = 44;

        public int NBolts { get; protected set; } = 88;


        /// <summary>
        /// 保存的加热策略名称
        /// </summary>
        public string HeatsProductName { get; protected set; } = "default";

        /// <summary>
        /// 连接到PLC?
        /// </summary>
        public bool IsConnectedWithPLC { get; set; }
        
        /// <summary>
        /// 上一次加热修改时间
        /// </summary>
        public DateTime LastChangedTime { get; set; } = DateTime.MinValue;


        #endregion
        #region 异常检测

        /// <summary>
        /// 烧了的加热棒
        /// </summary>
        public bool[] Bads { get; private set; }

        /// <summary>
        /// 当前风机启动中
        /// </summary>
        public bool HasFan { get; set; }

        /// <summary>
        /// 加热棒有电流
        /// </summary>
        public bool HasElectricity { get; set; }
        
        /// <summary>
        /// 当前正在检测的加热通道
        /// </summary>
        public int CheckNo { get; set; } = -2;
        /// <summary>
        /// 正在检测中
        /// </summary>
        public bool CheckEnable { get; set; }



        #endregion
        #endregion
        public FeedbackHeatServiceClient(UInt32 serverid) 
        {
            mServerID = serverid;

        }
        #region IFeedbackHeat 函数
        public void Apply()
        {
            var p = new FEEDBACK_OBJ_INTERFACE.Pack_Params()
            {
                step = Step,
                delay = Delay,
                hasCheck = HasCheck,
                HasCheckFilmVelocity = HasCheckFilmVelocity
            };
            string json = Newtonsoft.Json.JsonConvert.SerializeObject(p);
            FObjSys.Current.SetValueEx(
              mConn, mServerID, ID, 
              FEEDBACK_OBJ_INTERFACE.SET_PARAMS,
              Misc.Converter.StringToBytes(json));
        }
        /// <summary>
        /// 保存当前加热量
        /// </summary>
        /// <param name="productname"></param>
        public void SaveHeats(string productname)
        {
            string json = Newtonsoft.Json.JsonConvert.SerializeObject(productname);
            FObjSys.Current.CallFunctionEx(
               mConn, mServerID, ID, 
               FEEDBACK_OBJ_INTERFACE.CALL_SAVEHEATS, Misc.Converter.StringToBytes(json));
        }
        /// <summary>
        /// 获取保存的加热量文件列表 返回类型为 List&lt;string&gt;
        /// </summary>
        /// <param name="AsyncDelegate"></param>
        /// <param name="AsyncState"></param>
        public void GetHeatsFileList(AsyncCBHandler AsyncDelegate, object AsyncState)
        {
            CurrObjSys.CallFunctionEx(mConn, mServerID, ID,
                FEEDBACK_OBJ_INTERFACE.CALL_GETLIST,
                null,
                AsyncDelegate, AsyncState);
        }

        /// <summary>
        /// 删除 加热量文件
        /// </summary>
        /// <param name="productname"></param>
        public void DelHeatsFile(string productname) 
        {
            string json = Newtonsoft.Json.JsonConvert.SerializeObject(productname);

            FObjSys.Current.CallFunctionEx(
               mConn, mServerID, ID,
               FEEDBACK_OBJ_INTERFACE.CALL_DEL, Misc.Converter.StringToBytes(json));       
        }

        /// <summary>
        /// 加载 加热量文件 
        /// </summary>
        /// <param name="filename"></param>
        public void LoadHeatsFile(string productname) 
        {
            string json = Newtonsoft.Json.JsonConvert.SerializeObject(productname);
            FObjSys.Current.CallFunctionEx(
               mConn, mServerID, ID,
               FEEDBACK_OBJ_INTERFACE.CALL_LOADHEATS, Misc.Converter.StringToBytes(json));   
        }



        public void SetCheckEnable(bool enable) 
        {
            CheckEnable = enable;
            string json = Newtonsoft.Json.JsonConvert.SerializeObject(enable);
            FObjSys.Current.SetValueEx(
              mConn, mServerID, ID,
              FEEDBACK_OBJ_INTERFACE.SET_CHECKENABLE,
              Misc.Converter.StringToBytes(json));
        }
        public void SetEnable(bool enable) 
        {
           IsAuto = enable;
            string json = Newtonsoft.Json.JsonConvert.SerializeObject(enable);
            FObjSys.Current.SetValueEx(
              mConn, mServerID, ID,
              FEEDBACK_OBJ_INTERFACE.SET_ENABLE,
              Misc.Converter.StringToBytes(json));
        }
        #endregion

        #region IFObj
        public override void ConnectNotify(IFConn from)
        {
            mConn = from;

            IsConnected = from.IsConnected;
            if (from.IsConnected) 
            {

                FObjSys.Current.GetValueEx(mConn, mServerID, ID, FEEDBACK_OBJ_INTERFACE.GET_PARAMS);
                FObjSys.Current.GetValueEx(mConn, mServerID, ID, FEEDBACK_OBJ_INTERFACE.GET_STATE);
                
                FObjSys.Current.GetValueEx(mConn, mServerID, ID, FEEDBACK_OBJ_INTERFACE.GET_CHECKENABLE);
                FObjSys.Current.GetValueEx(mConn, mServerID, ID, FEEDBACK_OBJ_INTERFACE.GET_ENABLE);
                FObjSys.Current.GetValueEx(mConn, mServerID, ID, FEEDBACK_OBJ_INTERFACE.GET_ERROR);

                FObjSys.Current.GetValueEx(mConn, mServerID, ID, FEEDBACK_OBJ_INTERFACE.GET_PRODUCTNAME);
                
                FObjSys.Current.GetValueEx(mConn, mServerID, ID, FEEDBACK_OBJ_INTERFACE.GET_UNDOIDX);
                FObjSys.Current.SenseConfigEx(mConn, mServerID, ID, 0xffffffff, SENSE_CONFIG.ADD);
            }
        }




        public override void PushGetValue(IFConn from, uint srcid, ushort memid, byte[] infodata)
        {
            switch (memid) 
            {
                case FEEDBACK_OBJ_INTERFACE.GET_PARAMS:
                    {
                        string json = Misc.Converter.BytesToString(infodata);
                        var p = Newtonsoft.Json.JsonConvert.DeserializeObject<FEEDBACK_OBJ_INTERFACE.Pack_Params>(json);

                        Step = p.step;
                        Delay = p.delay;
                        
                        HasCheck = p.hasCheck;
                        HasCheckFilmVelocity = p.HasCheckFilmVelocity;
                    } break;
                case FEEDBACK_OBJ_INTERFACE.GET_STATE:
                    {
                        string json = Misc.Converter.BytesToString(infodata);
                        var p = Newtonsoft.Json.JsonConvert.DeserializeObject<FEEDBACK_OBJ_INTERFACE.Pack_Status>(json);

                        IsConnectedWithPLC = p.isConnected;
                        ChannelCnt = p.channelcnt;
                        NBolts = p.nbolts;
                        LastChangedTime = p.lastChangedTime;
                    } break;
                case FEEDBACK_OBJ_INTERFACE.GET_CHECKENABLE:
                    {
                        string json = Misc.Converter.BytesToString(infodata);
                        var p = Newtonsoft.Json.JsonConvert.DeserializeObject<bool>(json);
                        CheckEnable = p;

                    }break;
                case FEEDBACK_OBJ_INTERFACE.GET_ENABLE:
                    {
                        string json = Misc.Converter.BytesToString(infodata);
                        var p = Newtonsoft.Json.JsonConvert.DeserializeObject<bool>(json);
                        IsAuto = p;

                    } break;
                case FEEDBACK_OBJ_INTERFACE.GET_ERROR:
                    {
                        string json = Misc.Converter.BytesToString(infodata);
                        var p = Newtonsoft.Json.JsonConvert.DeserializeObject<FEEDBACK_OBJ_INTERFACE.Pack_Error>(json);

                        HasFan = p.hasFan;
                        HasElectricity = p.hasEletric;
                        CheckNo = p.checkno;
                        
                        Bads = p.bads;
                        NotifyPropertyChanged("Bads");
                    }break;


                case FEEDBACK_OBJ_INTERFACE.GET_PRODUCTNAME:
                    {
                        string json = Misc.Converter.BytesToString(infodata);
                        var p = Newtonsoft.Json.JsonConvert.DeserializeObject<string>(json);
                        HeatsProductName = p;
                    }break;

                case FEEDBACK_OBJ_INTERFACE.GET_UNDOIDX:
                    {
                        string json = Misc.Converter.BytesToString(infodata);

                        var p = Newtonsoft.Json.JsonConvert.DeserializeObject<FEEDBACK_OBJ_INTERFACE.Pack_Undo>(json);
                        
                        UndoIdx = p.undoIdx;
                        IsInUndo = p.isInUndo;
                        UndoTime = p.undoTime;
                    }
                    break;
            }
        }
        public override void PushCallFunction(IFConn from, uint srcid, uint magic, ushort funcid, byte[] retdata, object AsyncDelegate, object AsyncState)
        {
            switch (funcid) 
            {
                case FEEDBACK_OBJ_INTERFACE.CALL_GETLIST:
                    {
                        string json = Misc.Converter.BytesToString(retdata);
                        var p = Newtonsoft.Json.JsonConvert.DeserializeObject<List<string>>(json);

                        ((AsyncCBHandler)AsyncDelegate)(AsyncState, p);
                    } break;

            }
        }
        public override void PushInfo(IFConn from, uint srcid, ushort infoid, byte[] infodata)
        {
            PushGetValue(from, srcid, infoid, infodata);
        }
        public override void Dispose()
        {
            FObjSys.Current.SenseConfigEx(mConn, mServerID, ID, 0xffffffff, SENSE_CONFIG.REMOVE);
            base.Dispose();
        }
        #endregion


        #region INotifyPropertyChanged 成员

        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }
        #endregion





        #region 撤销

        /// <summary>
        /// undo 的序号
        /// </summary>
        public int UndoIdx { get; private set; }
        /// <summary>
        /// 在撤销中
        /// </summary>
        public bool IsInUndo { get; private set; }

        /// <summary>
        /// 这次撤销动作的时间
        /// </summary>
        public DateTime UndoTime { get; private set; }
        //UndoIdx--
        public void Undo()
        {
            FObjSys.Current.CallFunctionEx(
               mConn, mServerID, ID,
               FEEDBACK_OBJ_INTERFACE.CALL_UNDO, null
                );
        }

        //UndoIdx++
        public void Redo()
        {
            FObjSys.Current.CallFunctionEx(
               mConn, mServerID, ID,
               FEEDBACK_OBJ_INTERFACE.CALL_REDO, null
                );
        }

        #endregion


    }

}
