﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using FLY.FeedbackRenZiJia.OBJ_INTERFACE;
using System.Collections.ObjectModel;
using System.ComponentModel;
using FLY.FeedbackRenZiJia.Common;
using FLY.FeedbackRenZiJia.IService;
using PropertyChanged;

namespace FLY.FeedbackRenZiJia.Server
{
    /// <summary>
    /// 以前 正反转加热分离，所以有两个 HeatCell
    /// 现在不分正反转，只有一个HeatCell
    /// </summary>
    public class HeatCell : IHeatCell, Misc.ISaveToXml
    {
        #region IHeatCell 接口 成员变量

        #region 参数
        /// <summary>
        /// HeatOffset = ThickPercent * Kp
        /// </summary>
        public double Kp { get; set; } = 3;


        /// <summary>
        /// 自动对位
        /// </summary>
        public bool IsAutoONo { get; set; } = false;


        /// <summary>
        /// 自动对位的Kp
        /// </summary>
        public double AutoONoKp { get; set; } = 5;

        /// <summary>
        /// 平滑的阀值，只有超出阀值，才平滑
        /// </summary>
        public double ThresholdHeatSigma { get; set; } = 10;


        /// <summary>
        /// 自控时，不能调整总体上升下降
        /// </summary>
        public bool IsForbidUpDown { get; set; } = false;



        /// <summary>
        /// 厚度控制线 ， 厚度% 大于 厚度控制线 自动才会加热控制  
        /// </summary>
        public int CtrlLine { get; set; } = 2;

        /// <summary>
        /// 开启打散模式, 打散模式下，以打散度小为目标，控制2sigma在 控制线内
        /// </summary>
        public bool IsBreakUpMode { get; set; } = false;

        #endregion

        #region 状态


        /// <summary>
        /// 对应的100% 数据，单位是% 数量是BoltCnt, 数据肯定是完整的！！！！！
        /// </summary>
        [DoNotCheckEquality]
        public int[] ThickPercents { get; set; }



        /// <summary>
        /// 当前方向的加热策略，必须保证 0~100
        /// </summary>
        [DoNotCheckEquality]
        public int[] Heats { get; protected set; }


        private int Base { get; set; } = 0;




        /// <summary>
        /// 加热偏移量
        /// </summary>
        [DoNotCheckEquality]
        public int[] Offsets { get; protected set; }

        
        /// <summary>
        /// Heats+Offsets+Base
        /// </summary>
        [DoNotCheckEquality]
        public int[] PreHeats { get; protected set; }


        /// <summary>
        /// 加热量 每3个计算的sigma,最大值
        /// </summary>
        public double MaxHeatSigma { get; set; } = 0;


        /// <summary>
        /// 当前正在自动对位模式
        /// </summary>
        public bool IsInAutoONo { get; protected set; }


        #endregion
        #endregion
        
        public int ChannelCnt { get; set; }
        int BoltCnt;
        HeatBuf mHeatBuf;

        public Func<object,bool> PreHeatApplyEvent;
        public event Action<object> AfterHeatApplyEvent;
        public event Action<object> AfterClearOffsetsEvent;

        
        public HeatCell() 
        {


            Load();
            checkParamErr();
            Init2(88, 88);
            PropertyChanged += HeatCell_PropertyChanged;
            
        }
        void checkParamErr()
        {
            if (Kp < 1 || Kp > 10)
            {
                Kp = 3;
            }

            if (AutoONoKp < 1 || AutoONoKp > 10)
            {
                AutoONoKp = 5;
            }

            if (ThresholdHeatSigma < 1 || ThresholdHeatSigma > 50)
            {
                ThresholdHeatSigma = 10;
            }

            if (CtrlLine < 0 || CtrlLine > 10)
            {
                CtrlLine = 2;
            }
        }
        public void Init(HeatBuf heatBuf)
        {
            mHeatBuf = heatBuf;
        }
        public void Init2(int boltCnt, int channelCnt)
        {
            ChannelCnt = channelCnt;
            BoltCnt = boltCnt;

            Base = 0;
            MaxHeatSigma = 0;

            ThickPercents = new int[BoltCnt];
            Heats = new int[ChannelCnt];
            PreHeats = new int[ChannelCnt];
            Offsets = new int[ChannelCnt];
        }

        void HeatCell_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "IsAutoONo")
            {
                if (IsAutoONo == false)
                {
                    IsInAutoONo = false;
                }
            }
        }


       
        /// <summary>
        /// 自动模式！！！！！
        /// </summary>
        public void Auto()
        {
            if (mHeatBuf.mData.Count() == 0)
                return;

            if (IsAutoONo)
            {
                //判断当前加热全部为空， 且稳定性大于>0.7
                if (mHeatBuf.Stability == STABILITY.OK_CORREL)
                {
                    int[] offsets = new int[BoltCnt];
                    for (int i = 0; i < BoltCnt; i++)
                    {
                        int percent = ThickPercents[i];
                        offsets[i] = (int)Math.Round(percent * AutoONoKp);
                    }

                    Offsets = Common.MyMath.ZoomOut(offsets, BoltCnt / ChannelCnt);
                    UpdatePreHeats();//这句一定要加！！！！ 不然没法平滑，没法加热
                   
                    ModifyBase();
                    HeatApply();

                    IsInAutoONo = true;
                }
            }
            else
            {
                if (!mHeatBuf.IsUnstable())//稳定 与 待定时 加热
                {
                    int[] heatpercent = Common.MyMath.ZoomIn(Heats, BoltCnt / ChannelCnt);//放大加热数据量，与厚度一样
                    int avg = Misc.MyMath.Avg(heatpercent);
                    heatpercent = Common.MyMath.OffsetAvgBe0(heatpercent);//把加热量变成 平均值+偏移

                    int[] offsets = new int[BoltCnt];
                    for (int i = 0; i < BoltCnt; i++)
                    {
                        if (mHeatBuf.mBoltData[i].IsStable && //稳定
                            Misc.MyBase.ISVALIDATA(ThickPercents[i]) &&//数据非空
                            (Math.Abs(ThickPercents[i])> CtrlLine) )//在控制线已外
                        {
                            offsets[i] = (int)Math.Round(ThickPercents[i] * Kp);

                            //offsets[i] = mHeatBuf.mBoltData[i].Suggest(ThickPercents[i], heatpercent[i], offsets[i]);
                        }
                    }

                    if (IsBreakUpMode)//打断模式
                    {
                        //找到连续的块，当这个块的长度大于5， 给中间更大的offset
                        Misc.Range[] blocks = Common.MyMath.Break(ThickPercents);
                        if (blocks != null)
                        {
                            for (int i = 0; i < blocks.Count(); i++)
                            {
                                if (blocks[i].Width < 5)
                                    continue;

                                int idx = blocks[i].Mid;
                                if (idx < 0)
                                    idx += BoltCnt;
                                
                                if (mHeatBuf.mBoltData[idx].IsStable && //稳定
                                    Misc.MyBase.ISVALIDATA(ThickPercents[idx]) //数据非空
                                    )
                                {
                                    if(ThickPercents[i]>0)
                                        offsets[i] = (int)Math.Round((ThickPercents[i] + CtrlLine) * Kp);
                                    else
                                        offsets[i] = (int)Math.Round((ThickPercents[i] - CtrlLine) * Kp);

                                    offsets[i] = mHeatBuf.mBoltData[i].Suggest(ThickPercents[i], heatpercent[i], offsets[i]);
                                }
                            }
                        }
                    }


                    Offsets = Common.MyMath.ZoomOut(offsets, BoltCnt / ChannelCnt);//恢复为加热棒数量
                    UpdatePreHeats();//这句一定要加！！！！ 不然没法平滑，没法加热

                    Smooth();//平滑

                    if (!IsForbidUpDown)//禁止整体上移下降
                        ModifyBase();//自动整体上移下降

                    HeatApply();//执行加热
                }
            }
        }

        public void SetThickPercents(int[] thicks)
        {
            int[] thicks2 = thicks.Clone() as int[];
            MakeThicksFull(thicks);
            ThickPercents = Common.MyMath.GetPercent(thicks);
        }

        #region IHeatCell 接口 成员函数

        /// <summary>
        /// 把空的数据补齐
        /// </summary>
        /// <param name="thicks"></param>
        void MakeThicksFull(int[] thicks)
        {
            List<Misc.Range> emptys = new List<Misc.Range>();
            int start = -1;
            int end = -1;
            //变成数列
            for (int i = 0; i < BoltCnt; i++)
            {
                if (!Misc.MyBase.ISVALIDATA(thicks[i]))
                {
                    if (start == -1)
                    {
                        start = i;
                    }
                    end = i;
                }
                else
                {
                    if (start != -1)
                    {
                        emptys.Add(new Misc.Range() { Begin = start, End = end });
                        start = -1;
                    }
                }
            }
            if (start != -1)
            {
                emptys.Add(new Misc.Range() { Begin = start, End = end });
            }

            if (emptys.Count() > 0)
            {
                int e = emptys.Last().End;
                int b = emptys.First().Begin;
                if ((b == 0) && e == (BoltCnt - 1))
                {
                    //这是同一个
                    emptys.Last().End = emptys.First().End;
                    emptys.RemoveAt(0);
                }
                for (int i = 0; i < emptys.Count(); i++)
                {
                    Misc.Range r = emptys[i];
                    int idx = r.Begin - 1;
                    if (idx < 0)
                        idx += BoltCnt;

                    int b_thick = thicks[idx];

                    idx = r.End + 1;
                    if (idx >= BoltCnt)
                        idx -= BoltCnt;

                    int e_thick = thicks[idx];
                    //这个很重要， 当 结尾与开头重合， r.End < r.Begin, r.Width 会<0
                    int len = r.Width;
                    if (len < 0)
                        len += BoltCnt;

                    for (int j = 0; j < len; j++)
                    {
                        idx = r.Begin + j;
                        if (idx >= BoltCnt)
                            idx -= BoltCnt;
                        thicks[idx] = b_thick + (j + 1) * (e_thick - b_thick) / (len + 1);
                    }
                }
            }
        }


        /// <summary>
        /// 手动调整加热增量
        /// </summary>
        /// <param name="idx"></param>
        /// <param name="change"></param>
        public void ModifyOffset(int idx, int change)
        {
            if (idx < ChannelCnt && idx >= 0)
            {
                Offsets[idx] += change;
                UpdatePreHeats();
            }
            NotifyPropertyChanged("Offsets");
        }

        /// <summary>
        /// 参数是 baseheatoffset 的增量
        /// </summary>
        /// <param name="change"></param>
        public void ModifyBase(int change)
        {
            Base += change;
            UpdatePreHeats();
        }
        
        /// <summary>
        /// 调整BaseHeatOffset，令Heats 在范围内
        /// </summary>
        void ModifyBase()
        {
            int[] preheat = new int[ChannelCnt];
            for (int i = 0; i < ChannelCnt; i++)
                preheat[i] = Heats[i] + Offsets[i] + Base;

            int min = preheat.Min();
            int max = preheat.Max();
            int avg = (int)preheat.Average();

            int base_offset;
            if ((min < 0) && (max > 100))
            {
                //功率极限了
                //保存平均值为50
                base_offset = avg - 50;
            }
            else if (max > 100)
            {

                //整体往下降
                base_offset = max - 100;

                if (base_offset > (avg - 50))//移得太多
                    base_offset = (avg - 50);
            }
            else
            {
                base_offset = min;
                if (base_offset < 0)
                {
                    if (base_offset < (avg - 50))//升得太多
                        base_offset = (avg - 50);
                }
            }
            
            Base -= base_offset;

            UpdatePreHeats();
        }
        
        /// <summary>
        /// 清空HeatOffset
        /// </summary>
        public void ClearOffsets()
        {
            Base = 0;

            for (int i = 0; i < ChannelCnt; i++)
            {
                Offsets[i] = 0;
            }
            NotifyPropertyChanged("Offsets");
            UpdatePreHeats();

            AfterClearOffsetsEvent?.Invoke(this);
        }
        
        /// <summary>
        /// 清空Heats
        /// </summary>
        public void ClearPreHeats()
        {
            int[] heats = new int[ChannelCnt];
            Array.Clear(heats, 0, ChannelCnt);
            ModifyPreHeats(heats);
        }

        /// <summary>
        /// 调整PreHeats
        /// </summary>
        /// <param name="heats"></param>
        public void ModifyPreHeats(int[] heats)
        {
            
            if (heats.Count() != ChannelCnt)
                return;
            Base = 0;
            for (int i = 0; i < ChannelCnt; i++)
            {
                PreHeats[i] = heats[i];
                Offsets[i] = PreHeats[i] - Heats[i];
            }
            NotifyPropertyChanged("Offsets");
            NotifyPropertyChanged("PreHeats");
            UpdateHeatSigmas();
        }
        public void ModifyHeats(int[] heats)
        {
            if (heats.Count() != ChannelCnt)
                return;

            bool isChanged = false;
            Base = 0;
            for (int i = 0; i < ChannelCnt; i++)
            {
                if (Heats[i] != heats[i])
                {
                    isChanged = true;
                    Heats[i] = heats[i];
                    Offsets[i] = PreHeats[i] - Heats[i];
                }
            }
            if (isChanged)
            {
                NotifyPropertyChanged("Offsets");
                NotifyPropertyChanged("Heats");
            }
        }
        /// <summary>
        /// 利用测厚仪的数据，计算出HeatOffsets
        /// </summary>
        public void Cal()
        {
            int[] offsets = new int[BoltCnt];
            for (int i = 0; i < BoltCnt; i++)
            {
                int percent = ThickPercents[i];//100.0% 单位0.01%
                offsets[i] = (int)Math.Round(ThickPercents[i] * Kp);
            }
            Offsets = Common.MyMath.ZoomOut(offsets, BoltCnt / ChannelCnt);
            UpdatePreHeats();
        }

        #region PreHeats 平滑处理
        void UpdateHeatSigmas(int[] heats, out int[] heatsigmas)
        {
            heatsigmas = new int[ChannelCnt];
            for (int i = 0; i < ChannelCnt; i++)
            {
                int[] dat = new int[3];
                int index;
                for (int j = 0; j < 3; j++)
                {
                    index = i - 1 + j;
                    if (index < 0)
                        index += ChannelCnt;
                    else if (index >= ChannelCnt)
                        index -= ChannelCnt;

                    dat[j] = heats[index] * 100;
                }
                heatsigmas[i] = Misc.MyMath.Sigma(dat);
            }
        }
        void UpdateHeatSigmas()
        {
            int[] heatsigmas;
            UpdateHeatSigmas(PreHeats.ToArray(), out heatsigmas);
            MaxHeatSigma = heatsigmas.Max() / 100.0;
        }
        /// <summary>
        /// 平滑，不断修改PreHeats 直到HeatsSigma 合格
        /// </summary>
        public void Smooth()
        {
            int[] heats = PreHeats.ToArray();
            int[] heats2 = new int[ChannelCnt];
            int[] heatsigmas;
            bool hasChanged = false;
            bool hasChanged2 = false;
            // bool hasChanged3 = false;
            do
            {
                //1.更新sigma
                UpdateHeatSigmas(heats, out heatsigmas);

                //2.检测HeatsSigma， 哪里比较大，对该位置 滤波
                hasChanged = false;
                for (int i = 0; i < heats.Count(); i++)
                {
                    if (heatsigmas[i] > (ThresholdHeatSigma * 100)) //不行，要滤波！！！
                    {
                        //连续3个滤波
                        int[] dat = new int[3];
                        int index;
                        for (int j = 0; j < 3; j++)
                        {
                            index = i - 1 + j;
                            if (index < 0)
                                index += ChannelCnt;
                            else if (index >= ChannelCnt)
                                index -= ChannelCnt;

                            dat[j] = heats[index];
                        }
                        heats2[i] = (int)Math.Round(dat[0] * 0.33 + dat[1] * 0.34 + dat[2] * 0.33);
                        // (int)Math.Round(dat.Average());
                        if (heats2[i] != heats[i])
                            hasChanged = true;
                    }
                    else
                    {
                        heats2[i] = heats[i];
                    }
                }
                //3.有滤波行为，继续求循环，求sigma，判断。。。。。
                if (hasChanged)
                {
                    for (int i = 0; i < ChannelCnt; i++)
                    {
                        heats[i] = heats2[i];
                    }
                    hasChanged2 = true;
                }
            } while (hasChanged);

            if (hasChanged2)
            {
                for (int i = 0; i < ChannelCnt; i++)
                {
                    PreHeats[i] = heats[i];
                    Offsets[i] = PreHeats[i] - Heats[i] - Base;
                }
                NotifyPropertyChanged("PreHeats");
                NotifyPropertyChanged("Offsets");
                MaxHeatSigma = heatsigmas.Max() / 100.0;
            }
        }
        #endregion
        /// <summary>
        /// 加热输出应用
        /// </summary>
        public void HeatApply()
        {
            if (PreHeatApplyEvent != null)
            {
                if (!PreHeatApplyEvent(this))
                {
                    return;
                }
            }

            for (int i = 0; i < ChannelCnt; i++)
            {
                Offsets[i] = 0;

                int heat = PreHeats[i];
                if (heat < 0)
                {
                    //需要整体提升
                    heat = 0;
                }
                else if (heat > 100)
                {
                    //需要整体调低
                    heat = 100;
                }
                Heats[i] = heat;

            }
            NotifyPropertyChanged("Offsets");
            NotifyPropertyChanged("Heats");
            Base = 0;

            AfterHeatApplyEvent?.Invoke(this);
        }

        /// <summary>
        /// 修改Offsets, Base 后，执行PreHeats = Heats + Offsets + Base
        /// </summary>
        void UpdatePreHeats()
        {
            for (int i = 0; i < ChannelCnt; i++)
            {
                int preheat = Heats[i] + Offsets[i] + Base;

                if (preheat > 100)
                    preheat = 100;
                else if (preheat < 0)
                    preheat = 0;
                PreHeats[i] = preheat;
            }
            UpdateHeatSigmas();
            NotifyPropertyChanged("PreHeats");
        }

        public void Apply()
        {
            Save();
        }
        #endregion


        bool Load()
        {
            return Misc.SaveToXmlHepler.Load("heatcell.xml", this);
        }
        void Save()
        {
            Misc.SaveToXmlHepler.Save("heatcell.xml", this);
        }

        #region INotifyPropertyChanged 成员

        public event PropertyChangedEventHandler PropertyChanged;
        protected void NotifyPropertyChanged(string propertyname)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }


        #endregion
        public string[] GetSavePropertyNames()
        {
            return new string[]{
                "Kp",
                "AutoONoKp",
                "ThresholdHeatSigma",
                "IsForbidUpDown",
                "CtrlLine",
                "IsBreakUpMode"
            };
        }
    }
}
