﻿using FLY.Thick.Blowing.Client;
using FLY.Thick.Blowing.IService;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using ThickTcpUiInWindow;

namespace FLY.Thick.Blowing.UI.Fix.Client
{
    class PgBlowingVm : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        #region 属性
        /// <summary>
        /// 数据加载中......
        /// </summary>
        public bool IsBufListLoading { get; set; }
        /// <summary>
        /// 换向信号列表
        /// </summary>
        public ObservableCollection<SignData> SignList { get; } = new ObservableCollection<SignData>();
        /// <summary>
        /// 分区表
        /// </summary>
        public ObservableCollection<BoltMapCell> BoltMap { get; } = new ObservableCollection<BoltMapCell>();
        #endregion

        public BlowingFixServiceClient mRenZiJiaService;
        public BlowingDetectServiceClient mBDetect;

        public void Init(IPEndPoint serverep)
        {
            mRenZiJiaService = new BlowingFixServiceClient(FLY.Thick.Blowing.OBJ_INTERFACE.OBJ_INTERFACE_ID.RENZIJIA_ID);

            mBDetect = new BlowingDetectServiceClient(FLY.Thick.Blowing.OBJ_INTERFACE.OBJ_INTERFACE_ID.BDETECT_ID);

            FObjBase.FObjSys.Current.Connect_to_Another_OBJSys(
                serverep, mRenZiJiaService.ID, mBDetect.ID);

            UpdateMap();
            mRenZiJiaService.PropertyChanged += MRenZiJiaService_PropertyChanged;

        }
        void UpdateMap()
        {
            BoltMap.Clear();
            foreach (BoltMapCell m in mRenZiJiaService.Map)
            {
                BoltMap.Add(m);
            }
        }
        private void MRenZiJiaService_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Map")
            {
                UpdateMap();
            }
        }

        public void Download()
        {
            IsBufListLoading = true;
            mBDetect.GetSignList(
                (AsyncState, retData) =>
                {
                    GetSignListReponse reponse = retData as GetSignListReponse;
                    //reponse.firstbm

                    IsBufListLoading = false;


                    SignList.Clear();
                    //反转顺序
                    if (reponse.datas != null)
                    {
                        for (int i = 0; i < reponse.datas.Count(); i++)
                        {
                            int idx = reponse.datas.Count() - 1 - i;
                            SignList.Add(reponse.datas[idx]);
                        }
                    }

                }, null);
        }

        public void Apply()
        {
            if (Window_Password.Authorize(1))
            {
                string message;
                bool isvalid = CheckValid(out message);
                if (!isvalid)
                {
                    //异常出错
                    FLY.ControlLibrary.Window_WarningTip.Show("参数出错", message + "异常");
                }
                else
                {
                    mBDetect.Apply();
                    mRenZiJiaService.Apply();

                    FLY.ControlLibrary.Window_Tip.Show("应用成功",
                        null,
                        TimeSpan.FromSeconds(2));
                }
            }
        }

        bool CheckValid(out string message)
        {
            //检查参数是否合法
            if (mBDetect.RAngle < 100 || mBDetect.RAngle > 360)
            {
                message = "旋转总角度";
                return false;
            }

            if ((mBDetect.DefaultRPeriod.TotalMinutes < 3) || (mBDetect.DefaultRPeriod.TotalMinutes > 25))
            {
                message = "旋转时间";
                return false;
            }

            if ((mBDetect.RollPerimeter < 100) || (mBDetect.RollPerimeter > 1000))
            {
                message = "辊周长";
                return false;
            }
            if ((mBDetect.FilmLength < 0) || (mBDetect.FilmLength > 100))
            {
                message = "膜距离";
                return false;
            }

            if (mBDetect.AccDecTime.TotalMilliseconds > 5000)
            {
                message = "加减速时间";
                return false;
            }

            if (mBDetect.LimitSignTime.TotalSeconds > 30)
            {
                message = "撞信号总时间";
                return false;
            }


            if (mRenZiJiaService.ChannelCnt < 20 || mRenZiJiaService.ChannelCnt > 160)
            {
                message = "加热通道数";
                return false;
            }
            if (mRenZiJiaService.BPC < 1 || mRenZiJiaService.BPC > 4)
            {
                message = "分区数/加热通道数";
                return false;
            }
            if (mRenZiJiaService.OrgBoltNo < 1 || mRenZiJiaService.OrgBoltNo > (mRenZiJiaService.ChannelCnt * mRenZiJiaService.BPC))
            {
                message = "复位区号";
                return false;
            }

            if (mRenZiJiaService.EPCType == RenZiJiaFixEPCType.EPCA10)
            {
                if (mRenZiJiaService.SampleConsume.TotalSeconds < 1 || mRenZiJiaService.SampleConsume.TotalSeconds > 30)
                {
                    message = "温修时间";
                    return false;
                }
                if (mRenZiJiaService.SampleInterval.TotalMinutes <= 1 || mRenZiJiaService.SampleInterval.TotalMinutes > 120)
                {
                    message = "温修间隔";
                    return false;
                }
                if (mRenZiJiaService.BackEdgeWait.TotalSeconds < 1 || mRenZiJiaService.BackEdgeWait.TotalSeconds > 30)
                {
                    message = "边界等待";
                    return false;
                }
            }
            message = "成功";
            return true;
        }
        public void Dispose()
        {
            mRenZiJiaService.Dispose();
            mBDetect.Dispose();
        }

    }
}
