﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Net;

using FLY.Thick.BlowingScan.Client;
using FLY.Thick.Base.Client;
using ThickTcpUiInWindow;
using System.ComponentModel;
using FLY.Thick.BlowingScan.IService;
using System.Collections.ObjectModel;
using FLY.Thick.Blowing.Client;
using FLY.Thick.Blowing.IService;
using FObjBase;

namespace FLY.Thick.BlowingScan.UI.Client
{
    /// <summary>
    /// Page_Blowing.xaml 的交互逻辑
    /// </summary>
    public partial class PgBlowing : Page
    {
        PgBlowingViewModel viewModel;
        IPEndPoint serverep;

        public PgBlowing()
        {
            InitializeComponent();
        }
        
        public void Init(IPEndPoint serverep) 
        {
            this.serverep = serverep;
            viewModel = new PgBlowingViewModel();
            viewModel.Init(serverep);

            this.grid_blowing.DataContext = viewModel.mRenZiJiaService;
            this.grid_bDetect.DataContext = viewModel.mBDetect;
            this.grid_initparam.DataContext = viewModel.mInitParam;
            this.grid_viewModel.DataContext = viewModel;
        }


        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
        }


        private void Page_Unloaded(object sender, RoutedEventArgs e)
        {
            //viewModel.Dispose();
        }


        private void button_apply_click(object sender, RoutedEventArgs e)
        {
            viewModel.Apply();
        }

        private void button_buflist_Click(object sender, RoutedEventArgs e)
        {
            PgBufList p = new PgBufList();
            p.Init(serverep);
            NavigationService.Navigate(p);
        }

        private void button_download_Click(object sender, RoutedEventArgs e)
        {
            viewModel.Download();
        }

        private void BackButtonClick(object sender, RoutedEventArgs e)
        {
            viewModel.Dispose();
        }
    }

    public class PgBlowingViewModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        #region 属性
        /// <summary>
        /// 数据加载中......
        /// </summary>
        public bool IsBufListLoading { get; set; }
        /// <summary>
        /// 换向信号列表
        /// </summary>
        public ObservableCollection<SignData> SignList { get; } = new ObservableCollection<SignData>();
        /// <summary>
        /// 分区表
        /// </summary>
        public ObservableCollection<BoltMapCell> BoltMap { get; } = new ObservableCollection<BoltMapCell>();
        #endregion

        public BlowingScanServiceClient mRenZiJiaService;
        public BlowingDetectServiceClient mBDetect;
        public InitParamServiceClient mInitParam;
        public void Init(IPEndPoint serverep)
        {
            mRenZiJiaService = new BlowingScanServiceClient(FLY.Thick.BlowingScan.OBJ_INTERFACE.OBJ_INTERFACE_ID.RENZIJIA_ID);
            mBDetect = new BlowingDetectServiceClient(FLY.Thick.BlowingScan.OBJ_INTERFACE.OBJ_INTERFACE_ID.BDETECT_ID);
            mInitParam = new InitParamServiceClient(FLY.Thick.BlowingScan.OBJ_INTERFACE.OBJ_INTERFACE_ID.INITPARAM_ID);

            FObjBase.FObjSys.Current.Connect_to_Another_OBJSys(
                serverep, mRenZiJiaService.ID, mBDetect.ID, mInitParam.ID);

            UpdateMap();
            mRenZiJiaService.PropertyChanged += MRenZiJiaService_PropertyChanged;

        }
        void UpdateMap()
        {
            BoltMap.Clear();
            foreach (BoltMapCell m in mRenZiJiaService.Map)
            {
                BoltMap.Add(m);
            }
        }
        private void MRenZiJiaService_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Map")
            {
                UpdateMap();
            }
        }

        public void Download()
        {
            IsBufListLoading = true;
            mBDetect.GetSignList(
                (AsyncState, retData) =>
                {
                    GetSignListReponse reponse = retData as GetSignListReponse;
                    //reponse.firstbm

                    IsBufListLoading = false;


                    SignList.Clear();
                    //反转顺序
                    if (reponse.datas != null)
                    {
                        for (int i = 0; i < reponse.datas.Count(); i++)
                        {
                            int idx = reponse.datas.Count() - 1 - i;
                            SignList.Add(reponse.datas[idx]);
                        }
                    }

                }, null);
        }

        public void Apply()
        {
            if (Window_Password.Authorize(1))
            {
                string message;
                bool isvalid = CheckValid(out message);
                if (!isvalid)
                {
                    //异常出错
                    FLY.ControlLibrary.Window_WarningTip.Show("参数出错", message + "异常");
                }
                else
                {
                    mBDetect.Apply();
                    mRenZiJiaService.Apply();

                    FLY.ControlLibrary.Window_Tip.Show("应用成功",
                        null,
                        TimeSpan.FromSeconds(2));
                }
            }
        }

        bool CheckValid(out string message)
        {
            //检查参数是否合法
            if (mBDetect.RAngle < 100 || mBDetect.RAngle > 360)
            {
                message = "旋转总角度";
                return false;
            }

            if ((mBDetect.DefaultRPeriod.TotalMinutes < 3) || (mBDetect.DefaultRPeriod.TotalMinutes > 25))
            {
                message = "旋转时间";
                return false;
            }

            if ((mBDetect.RollPerimeter < 100) || (mBDetect.RollPerimeter > 1000))
            {
                message = "辊周长";
                return false;
            }
            if ((mBDetect.FilmLength < 0) || (mBDetect.FilmLength > 100))
            {
                message = "膜距离";
                return false;
            }

            if (mBDetect.AccDecTime.TotalMilliseconds > 5000)
            {
                message = "加减速时间";
                return false;
            }

            if (mBDetect.LimitSignTime.TotalSeconds > 30)
            {
                message = "撞信号总时间";
                return false;
            }


            if (mRenZiJiaService.ChannelCnt < 20 || mRenZiJiaService.ChannelCnt > 160)
            {
                message = "加热通道数";
                return false;
            }
            if (mRenZiJiaService.BPC < 1 || mRenZiJiaService.BPC > 4)
            {
                message = "分区数/加热通道数";
                return false;
            }
            if (mRenZiJiaService.OrgBoltNo < 1 || mRenZiJiaService.OrgBoltNo > (mRenZiJiaService.ChannelCnt * mRenZiJiaService.BPC))
            {
                message = "复位区号";
                return false;
            }


            message = "成功";
            return true;
        }
        public void Dispose()
        {
            mRenZiJiaService.Dispose();
            mBDetect.Dispose();
            mInitParam.Dispose();
        }

    }

    public class PgBlowingViewModel_UnitTest : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        #region 属性
        /// <summary>
        /// 数据加载中......
        /// </summary>
        public bool IsBufListLoading { get; set; }
        /// <summary>
        /// 换向信号列表
        /// </summary>
        public ObservableCollection<SignData> SignList { get; } = new ObservableCollection<SignData>();
        /// <summary>
        /// 分区表
        /// </summary>
        public ObservableCollection<BoltMapCell> BoltMap { get; } = new ObservableCollection<BoltMapCell>();
        #endregion
    }
    public class BoltLengthMultiConverter : IMultiValueConverter
    {

        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            //FilmVelocity
            //RenZiJiaPeriod
            //NBolts
            if (values.Count() != 3)
            {
                return null;
            }
            if ((values[0] is double) && (values[1] is TimeSpan) && (values[2] is int))
            {
                double FilmVelocity = (double)values[0];
                TimeSpan RenZiJiaPeriod = (TimeSpan)values[1];
                int NBolts = (int)values[2];
                return RenZiJiaPeriod.TotalMinutes / NBolts * FilmVelocity;
            }
            return null;

        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
    /// <summary>
    /// 旋转架及膜位置检测模拟
    /// </summary>
    public class BlowingDetect_UnitTest : INotifyPropertyChanged
    {
        #region 参数

        /// <summary>
        /// 离开限位 到 撞下一个限位 的 旋转架转动总角度 单位°
        /// </summary>
        public double RAngle { get; set; }

        /// <summary>
        /// 人字架 旋转1周 设置 时间。 
        /// 刚开机时，RenZiJiaPeriod = DefaultRPeriod；
        /// 异常时，RenZiJiaPeriod = DefaultRPeriod；
        /// </summary>
        public TimeSpan DefaultRPeriod { get; set; }

        /// <summary>
        /// 加速减速时间，  默认1s  加速减速时间 * 2 +正反转冷却时间 = 撞限位 到 离开限位 的时间
        /// </summary>
        public TimeSpan AccDecTime { get; set; }

        /// <summary>
        /// 信号0 撞2次。 这个能自动设置。 两个撞同一信号 间隔30秒内，算一个换向信号。 
        /// 当为2次撞时：  StartTime = 第1次撞的StartTime， EndTime = 第2次撞的EndTime
        /// </summary>
        public bool IsSign0Double { get; set; }

        /// <summary>
        /// 信号1 撞2次。 这个能自动设置。 两个撞同一信号 间隔30秒内，算一个换向信号。 
        /// 当为2次撞时：  StartTime = 第1次撞的StartTime， EndTime = 第2次撞的EndTime
        /// </summary>
        public bool IsSign1Double { get; set; }

        /// <summary>
        /// 信号模式， 短信号模式（只撞一下），长信号模式（只要是转着，信号就长亮，直到换向）
        /// </summary>
        public BlowingSignType SignType { get; set; }

        /// <summary>
        /// 撞限位 到 离开限位 的时间， 需要初始值，以后测量出来的
        /// </summary>
        public TimeSpan LimitSignTime { get; set; }

        /// <summary>
        /// 人字架到测厚仪膜长 单位m
        /// 当旋转架为立式时， 膜长为人字架复位时，的最短膜距离
        /// </summary>
        public double FilmLength { get; set; }

        /// <summary>
        /// 辊周长，单位mm
        /// </summary>
        public double RollPerimeter { get; set; }

        /// <summary>
        /// 转向信号100ms 滤波
        /// </summary>
        public bool IsSignFilter { get; set; }

        /// <summary>
        /// 缺少信号模式
        /// 正反信号线接得不好，有时候会丢失了。 
        /// 当前的旋转时间是上一次的1.1倍时，在以前的位置增加信号！！！！
        /// 当有连续两次相同的信号，且信号差刚好是旋转时间的1.9~2.1倍时，证明真的是缺少信号
        /// </summary>
        public bool IsLackSignMode { get; set; }

        /// <summary>
        /// 旋转架为立式，测厚仪安装在二牵引前面，膜距离 与 旋转角度 一起变化， 旋转角度越大 膜距离越大
        /// </summary>
        public bool Is3D { get; set; }



        #endregion

        #region 状态
        /// <summary>
        /// 当前旋转架旋转角度
        /// </summary>
        public double Angle { get; set; }

        /// <summary>
        /// 当前方向
        /// </summary>
        public Misc.DIRECTION Direction { get; set; }

        /// <summary>
        /// 旋转次数
        /// </summary>
        public int RotationCnt { get; set; }

        /// <summary>
        /// 最新的限位信号序号
        /// </summary>
        public int LastLimitNo { get; set; }

        /// <summary>
        /// 最后一次撞限位，或离开限位信号 到当前已经消耗的时间
        /// </summary>
        public TimeSpan PastTime { get; set; }

        /// <summary>
        /// 当前线速度,单位 m/min
        /// </summary>
        public double FilmVelocity { get; set; }

        /// <summary>
        /// 离开限位 到 撞下一个限位  的 旋转架转动时间， 需要初始值，以后测量出来的
        /// </summary>
        public TimeSpan RenZiJiaPeriod { get; set; }

        /// <summary>
        /// 正反转冷却时间， 加速减速时间 * 2 +正反转冷却时间 = 撞限位 到 离开限位 的时间 
        /// </summary>
        public TimeSpan SwapCoolTime { get; set; }

        /// <summary>
        /// 缓冲区拥有数据的时间长度，1s更新一次
        /// </summary>
        public TimeSpan BufTotalTime { get; set; }

        public event PropertyChangedEventHandler PropertyChanged;
        #endregion

        #region 功能
        /// <summary>
        /// 参数应用
        /// </summary>
        public void Apply() { }

        /// <summary>
        /// 获取 信号列表
        /// </summary>
        /// <param name="AsyncDelegate">retdata = GetBufListReponse</param>
        /// <param name="AsyncState"></param>
        public void GetSignList(AsyncCBHandler AsyncDelegate, object AsyncState) { }

        /// <summary>
        /// 获取 辊信号列表
        /// </summary>
        /// <param name="begin">开始时间</param>
        /// <param name="AsyncDelegate">retdata = GetRollListReponse</param>
        /// <param name="AsyncState"></param>
        public void GetRollList(DateTime begin, AsyncCBHandler AsyncDelegate, object AsyncState) { }

        /// <summary>
        /// 获取 转向信号列表
        /// </summary>
        /// <param name="begin">开始时间</param>
        /// <param name="AsyncDelegate">retdata = GetRollListReponse</param>
        /// <param name="AsyncState"></param>
        public void GetLimitList(DateTime begin, AsyncCBHandler AsyncDelegate, object AsyncState) { }

        /// <summary>
        /// 获取 立式旋转架 的 膜距离增量
        /// </summary>
        /// <param name="AsyncDelegate">retdata = GetFilmLength3DReponse</param>
        /// <param name="AsyncState"></param>
        public void GetFilmLength3D(AsyncCBHandler AsyncDelegate, object AsyncState) { }

        /// <summary>
        /// 设置 立式旋转架 的 膜距离增量
        /// </summary>
        /// <param name="data"></param>
        public void SetFilmLength3D(List<double> data) { }
        #endregion
    }
}
