﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.ComponentModel;
using System.Collections.ObjectModel;

using System.Net;


using ThickTcpUiInWindow;
using FObjBase;
using FLY.Thick.BlowingScan.IService;
using FLY.Thick.BlowingScan.Client;
using FLY.Thick.BlowingScan.Common;

namespace FLY.Thick.BlowingScan.UI.Client
{
    /// <summary>
    /// Page_ProfileBlowing.xaml 的交互逻辑
    /// </summary>
    public partial class PgProfileBlowing : Page, INotifyPropertyChanged
    {
        public ObservableCollection<string> mList = new ObservableCollection<string>();
        public BlowingProfileServiceClient mProfile = null;

        public PgProfileBlowing()
        {
            InitializeComponent();
        }
        public void Init(IPEndPoint serverep)
        {
            mProfile = new BlowingProfileServiceClient(FLY.Thick.Blowing.OBJ_INTERFACE.OBJ_INTERFACE_ID.BFPROFILE_ID);

            FObjBase.FObjSys.Current.Connect_to_Another_OBJSys(
                serverep, mProfile.ID);

            this.DataContext = mProfile.Param;
            this.listview_profile.ItemsSource = mList;
            
            mProfile.GetList(
                new AsyncCBHandler(
                    delegate(object AsyncState, object retData)
                    {
                        List<string> list = (List<string>)retData;
                        mList.Clear();
                        for (int i = 0; i < list.Count(); i++)
                            mList.Add(list[i]);

                        if (mList.Contains(mProfile.Param.PName))
                        {
                            listview_profile.SelectedItem = mList.First(s => s == mProfile.Param.PName);
                        }
                    }), null);
        }


        private void Page_Loaded(object sender, RoutedEventArgs e)
        {
        }

        private void button_read_Click(object sender, RoutedEventArgs e)
        {
            if (listview_profile.SelectedItem != null)
            {
                string productname = listview_profile.SelectedItem as string;
                mProfile.Read(
                    productname,
                    new AsyncCBHandler(
                    delegate(object AsyncState, object retData)
                    {
                        BlowingProfileParam p = (BlowingProfileParam)retData;
                        Misc.PropertiesManager.CopyTo(p, mProfile.Param);
                        
                    }), null);
            }
        }

        private void button_apply_Click(object sender, RoutedEventArgs e)
        {
            //有效性检查
            if (mProfile.Param.Target <= 0)
            {
                FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "目标值<=0 异常");
                return;
            }

            if (mProfile.Param.TolerancePercent <= 0)
            {
                FLY.ControlLibrary.Window_WarningTip.Show("参数出错", "公差 % <=0 异常");
                return;
            }

            mProfile.Apply();
            if (!mList.Contains(mProfile.Param.PName))
            {
                mList.Add(mProfile.Param.PName);
            }
            FLY.ControlLibrary.Window_Tip.Show("应用 & 保存成功",
                mProfile.Param.PName,
                TimeSpan.FromSeconds(2));

        }

        private void button_del_Click(object sender, RoutedEventArgs e)
        {
            if (listview_profile.SelectedItem != null)
            {
                string productname = listview_profile.SelectedItem as string;
                mProfile.Del(productname);
                mList.Remove(productname);
                FLY.ControlLibrary.Window_Tip.Show("删除成功",
                    productname,
                    TimeSpan.FromSeconds(2));
            }
        }

        private void button_abhelper_Click(object sender, RoutedEventArgs e)
        {

            Window_AHelper w = new Window_AHelper();
            w.Init(mProfile.Param.K);
            w.Owner = FLY.ControlLibrary.COMMON.GetWindow(this);
            if (w.ShowDialog() == true)
            {
                mProfile.Param.K = w.A;
            }
        }


        private void Page_Unloaded(object sender, RoutedEventArgs e)
        {
            mProfile.Dispose();
        }
        #region INotifyPropertyChanged 成员

        public event PropertyChangedEventHandler PropertyChanged;


        #endregion
    }
}
