﻿using FLY.OBJComponents.Common;
using FLY.OBJComponents.IService;
using FObjBase;
using Misc;
using SQLite;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FLY.Weight.Server.Model
{
    public class BufferMix<TDb> : IBufferAdd<Lc_Mix>
        where TDb : IDbBase, new()
    {
        DBTable<TDb> dbTable;
        SQLiteHelper sqliteHelper;
        Func<Lc_Mix, TDb> mapLc2Db;
        Func<TDb, Lc_Mix> mapDb2Lc;

        public int NewestID => Count - 1;

        public int Count { get; private set; }

        /// <summary>
        /// 层序号
        /// </summary>
        public int Index { get; private set; }
        /// <summary>
        /// 不能设置，设了也没有用
        /// </summary>
        public int Capacity { get; set; } = int.MaxValue;

        public event NotifyBufferChangedEventHandler<Lc_Mix> BufferChanged;
        public event PropertyChangedEventHandler PropertyChanged;


        public BufferMix()
        {

        }
        public void Init(DBTable<TDb> dbTable, int index,
            Func<Lc_Mix, TDb> mapLc2Db, Func<TDb, Lc_Mix> mapDb2Lc)
        {
            this.Index = index;
            this.dbTable = dbTable;
            this.sqliteHelper = dbTable.sqliteHelper;
            this.mapLc2Db = mapLc2Db;
            this.mapDb2Lc = mapDb2Lc;
            Load();
        }
        void Load()
        {
            var n = sqliteHelper.ExecuteScalar($"SELECT COUNT(*) FROM {dbTable.TableName} WHERE Idx={Index}");
            int count = System.Convert.ToInt32(n);
            Count = count;
        }

        public void Add(Lc_Mix lc)
        {
            //添加 数据库必要的 field
            lc.ID = dbTable.FreeID;
            lc.Idx = Index;
            //SQLs
            List<string> sqls = new List<string>();
            sqls.Add(SQLiteHelper.GetInsertCommandText(mapLc2Db(lc)));
            sqliteHelper.QueryTranAsync(sqls);

            Count++;
        }
        public void AddRange(IEnumerable<Lc_Mix> lcs)
        {
            //SQLs
            List<string> sqls = new List<string>();

            foreach (var lc in lcs)
            {
                //添加 数据库必要的 field
                lc.ID = dbTable.FreeID;
                lc.Idx = Index;
                sqls.Add(SQLiteHelper.GetInsertCommandText(mapLc2Db(lc)));
            }
            sqliteHelper.QueryTranAsync(sqls);

            Count += lcs.Count();
        }

        /// <summary>
        /// 清空全部数据
        /// </summary>
        public void Reset()
        {
            //SQLs
            List<string> sqls = new List<string>();
            sqls.Add($"DELETE FROM {dbTable.TableName} WHERE Idx={Index}");
            sqliteHelper.QueryTranAsync(sqls);
            Count = 0;
        }

        /// <summary>
        /// 获取指定位置的N个数据
        /// </summary>
        /// <param name="last_id">最后的ID</param>
        /// <param name="count">数量</param>
        /// <param name="asyncCB">回调</param>
        /// <param name="asyncContext">回调里面的上下文</param>
        public async void GetRecord(int last_id, int count, AsyncCBHandler asyncCB, object asyncContext)
        {
            if (Count == 0)
            {
                asyncCB(asyncContext, new GetRecordReponse<Lc_Mix>()
                {
                    LastID = 0,
                    Items = null
                });
                return;
            }

            await Task.Factory.StartNew(() => {
                var reponse = new GetRecordReponse<Lc_Mix>();

                var dbs = dbTable.Find($"WHERE Idx={Index} ORDER BY ID LIMIT {last_id - count + 1},{count}");
                reponse.LastID = last_id;
                List<Lc_Mix> lcs = new List<Lc_Mix>();
                foreach (var db in dbs)
                {
                    lcs.Add(mapDb2Lc(db));
                }
                reponse.Items = lcs;

                asyncCB(asyncContext, reponse);
            });

        }

        /// <summary>
        /// 获取最新的N个数据
        /// </summary>
        /// <param name="count">数量</param>
        /// <param name="asyncCB">回调</param>
        /// <param name="asyncContext">回调里面的上下文</param>
        public void GetRecord(int count, AsyncCBHandler asyncCB, object asyncContext)
        {
            GetRecord(NewestID, count, asyncCB, asyncContext);
        }
    }
}
