﻿
using FLY.OBJComponents.Server;
using FLY.OBJComponents.Server.Model;

using FObjBase;
using SQLite;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace FLY.Weight.Server.Model
{
    /// <summary>
    /// 数据库 写操作
    /// </summary>
    public class HistoryDb
    {
        DbModel dbModel;
        public BufferSQLite<Lc_Flow, Db_Flow> FlowBuffer;
        public BufferMix<Db_Mix>[] MixBuffers;
        public BufferMix<Db_RollMix>[] RollMixBuffers;
        public BufferError ErrorBuffer;
        /// <summary>
        /// 
        /// </summary>
        /// <param name="dBModel"></param>
        public void Init(DbModel dBModel, int weighterCnt)
        {
            this.dbModel = dBModel;
            FlowBuffer = new BufferSQLite<Lc_Flow, Db_Flow>();
            FlowBuffer.Init(dBModel.TbFlow,
                (lc) => Lc_AutoMapperProfile.Mapper.Map<Db_Flow>(lc),
                (db) => Lc_AutoMapperProfile.Mapper.Map<Lc_Flow>(db)
                );

            MixBuffers = new BufferMix<Db_Mix>[weighterCnt];
            RollMixBuffers = new BufferMix<Db_RollMix>[weighterCnt];
            for (int i = 0; i < weighterCnt; i++)
            {
                MixBuffers[i] = new BufferMix<Db_Mix>();
                MixBuffers[i].Init(dBModel.TbMix, i,
                    (lc) => Lc_AutoMapperProfile.Mapper.Map<Lc_Mix, Db_Mix>(lc),
                    (db) => Lc_AutoMapperProfile.Mapper.Map<Db_Mix, Lc_Mix>(db)
                    );

                RollMixBuffers[i] = new BufferMix<Db_RollMix>();
                RollMixBuffers[i].Init(dBModel.TbRollMix, i,
                    (lc) => Lc_AutoMapperProfile.Mapper.Map<Lc_Mix, Db_RollMix>(lc),
                    (db) => Lc_AutoMapperProfile.Mapper.Map<Db_RollMix, Lc_Mix>(db)
                    );
            }
            ErrorBuffer = new OBJComponents.Server.BufferError();
            ErrorBuffer.Init(dBModel.TbError);
        }

        /// <summary>
        /// 按时间删除数据库
        /// </summary>
        /// <param name="month"></param>
        public void KeepDBSize(int month)
        {
            if (month <= 3)
                month = 3;

            DateTime del_time = DateTime.Now - TimeSpan.FromDays(month * 30);
            string det_time_str = del_time.ToStringOfSQLiteFieldType();
            List<string> sqls = new List<string>();
            sqls.Add(
                $"DELETE FROM {dbModel.TbFlow.TableName}" +
                $" WHERE Time < {det_time_str}");

            sqls.Add(
                $"DELETE FROM {dbModel.TbMix.TableName}" +
                $" WHERE Time < {det_time_str}");

            sqls.Add(
                $"DELETE FROM {dbModel.TbRollMix.TableName}" +
                $" WHERE Time < {det_time_str}");

            sqls.Add(
                $"DELETE FROM {dbModel.TbError.TableName}" +
                $" WHERE Time < {det_time_str}");

            dbModel.sqliteHelper.QueryTran(sqls);

        }

        public void AddFlow(
            Lc_Flow flow
            )
        {
            FlowBuffer.Add(flow);
        }
        public void AddFlowRange(IEnumerable<Lc_Flow> flows) 
        {
            FlowBuffer.AddRange(flows);
        }
        public void AddMix(int index,
            Lc_Mix mix
            )
        {
            MixBuffers[index].Add(mix);
        }

        public void AddRollMix(int index,
            Lc_Mix mix
            )
        {
            RollMixBuffers[index].Add(mix);
        }
    }
}
