﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using OfficeOpenXml;
using System.ComponentModel;
using System.Text.RegularExpressions;

namespace Autogen_WS
{
    public class DataInfos:INotifyPropertyChanged
    {
        /// <summary>
        /// 输出的C#对象属性表
        /// </summary>
        List<WS_ItemInfo> infos = new List<WS_ItemInfo>();
        /// <summary>
        /// 设备EP
        /// </summary>
        DeviceAddr mDeviceAddrs = new DeviceAddr();
        /// <summary>
        /// 地址段映射
        /// </summary>
        DevMapperRuleTable mDevMapperRuleTable = new DevMapperRuleTable();

        int plc_cnt =0;

        #region 属性
        /// <summary>
        /// 进度
        /// </summary>
        public double Progress { get; set; }

        /// <summary>
        /// 输出的xml文件
        /// </summary>
        public string XlsxFile { get; set; }



        /// <summary>
        /// C#文件地址
        /// </summary>
        public string CSharpPath
        {
            get;
            set;
        }

        #endregion
        public DataInfos()
        {

        }


        public bool LoadXlsxFile() 
        {
            //加载地址
            if (!mDeviceAddrs.Load(XlsxFile)) return false;
            //加载转换表
            if (!mDevMapperRuleTable.Load(XlsxFile)) return false;
            //加载 & 转换地址
            return LoadXlsxFile(XlsxFile);
        }

        /// <summary>
        /// 分析 "D(6120)" 分拆为寄存器类型, 寄存器地址
        /// </summary>
        /// <param name="plc_addr">寄存器地址字符串</param>
        /// <param name="type">寄存器类型</param>
        /// <param name="addr">寄存器地址</param>
        /// <returns>是否成功</returns>
        bool AnalysisAddr(string plc_addr, out string type, out int addr)
        {

            type = null;
            addr = 0;
            //string plc_addr = "设备1_读写4WUB3821";
            //Console.WriteLine("查找对象=" + plc_addr);


            Regex r = new Regex(@"(\w)\((\d+)\)");//分析 D(6120)
            Match m = r.Match(plc_addr);
            if (!m.Success)
                return false;
            string plc_vt = m.Groups[1].Value;
            int plc_a = int.Parse(m.Groups[2].Value);

            //查表
            var cells = from _cell in mDevMapperRuleTable.Tables where _cell.PLCVarType == plc_vt select _cell;
            if (cells.Count() == 0)
                return false;


            type = cells.First().VarType;
            //特别的， M0~...   D0~.... 是连续的
            //但 X0~X7 ,X10~X17, Y0~Y7 ,Y10~Y17, 不是连续的
            if (plc_vt == "X" || plc_vt == "Y")
            {
                plc_a = ((int)(plc_a / 10)) * 8 + plc_a % 10;
            }
            addr = (plc_a - cells.First().PLCAddr1st) + cells.First().Addr1st;

            return true;

        }

        /// <summary>
        /// 加载 HMI-地址表.xlsm
        /// </summary>
        /// <param name="xlsxfile"></param>
        /// <returns></returns>
        public bool LoadXlsxFile(string xlsxfile)
        {
            int i_propDescription = 2;//描述
            int i_propName = 3;//属性名
            int i_typeName = 4;//类型
            int i_scale = 5;//放大
            int i_canWrite = 6;//可写
            int i_plc_addr = 7;//HMI寄存器

            string str;

            infos.Clear();
            Progress = 0;
            using (ExcelPackage packages = new ExcelPackage(new FileInfo(xlsxfile)))
            {
                using (ExcelWorksheet sheet = packages.Workbook.Worksheets["参数"])
                {
                    int iRow = 2;
                    while (true)
                    {
                        WS_ItemInfo ii = new WS_ItemInfo();

                        try
                        {
                            ii.propDescription = sheet.Cells[iRow, i_propDescription].Value.ToString();
                        }
                        catch
                        {
                            ii.propDescription = null;
                        }


                        try
                        {
                            ii.propName = sheet.Cells[iRow, i_propName].Value.ToString();
                        }
                        catch
                        {
                            ii.propName = null;//没有名称异常
                            break;
                        }



                        try
                        {
                            ii.typeName = sheet.Cells[iRow, i_typeName].Value.ToString();
                        }
                        catch
                        {
                            ii.typeName = null;//没有类型异常
                            break;
                        }


                        try
                        {
                            string scale_str = sheet.Cells[iRow, i_scale].Value.ToString();
                            if (!string.IsNullOrEmpty(scale_str))
                            {
                                double d;
                                if (double.TryParse(scale_str, out d))
                                {
                                    ii.scale = d;
                                }
                            }
                        }
                        catch
                        {

                        }
                        ii.plc_index = 0;
                        ii.objectName = "Item";
                        //propertyname 不能有重复
                        if (!infos.Any((info) => { return info.propName == ii.propName; }))
                        {
                            string plc_addr;
                            try
                            {
                                plc_addr = sheet.Cells[iRow, i_plc_addr].Value.ToString();
                            }
                            catch
                            {
                                //空的
                                goto _next;
                            }

                            if (!AnalysisAddr(plc_addr, out ii.reg_type, out ii.mb_addr))
                            {
                                goto _next;
                            }

                            infos.Add(ii);

                            ii = ii.Clone();
                        }
                        _next:
                        iRow++;
                        Progress += 1;
                        if (Progress >= 100)
                            Progress = 50;
                    }
                    Progress = 100;
                    return true;
                }
            }
        }




        void Gen_CS(StreamWriter sw)
        {
           

            Dictionary<string, WS_ItemInfo> d = new Dictionary<string, WS_ItemInfo>();

            foreach (WS_ItemInfo ii in infos)
            {
                if (d.ContainsKey(ii.propName)) continue;
                d.Add(ii.propName, ii);
            }
            sw.WriteLine("        #region 辅助代码生成");
            foreach (KeyValuePair<string, WS_ItemInfo> sii in d)
            {
                WS_ItemInfo info = sii.Value;
                sw.WriteLine("        /// <summary>");
                sw.WriteLine("        /// {0}", info.propDescription);
                sw.WriteLine("        /// </summary>");
                sw.Write("        public {0} {1}", info.GetPropertyType(), info.propName);
                sw.WriteLine(" { get; set; }");
                sw.WriteLine();
            }
            sw.WriteLine("        #endregion");
        }

        public void Gen_CS()
        {
            //找到 #region 辅助代码生成 与最接近的 #endregion

            //StringBuilder sb = new StringBuilder();
            List<String> file_content_header = new List<string>();
            List<String> file_content_tailer = new List<string>();
            int state = 0;
            int region_cnt = 0;

            string weightc_new_path = Path.Combine(Path.GetDirectoryName(XlsxFile), "Generated", Path.GetFileName(CSharpPath));

            StreamWriter sw = new StreamWriter(weightc_new_path, false, Encoding.UTF8);
            StreamReader sr = new StreamReader(CSharpPath);

            //把整个文件读到 sb中
            while (!sr.EndOfStream)
            {
                string line = sr.ReadLine();
                switch (state)
                {
                    case 0:
                        if (line.Contains("#region 辅助代码生成"))
                        {
                            //找到了
                            state = 1;
                            region_cnt = 1;
                        }
                        else
                        {
                            sw.WriteLine(line);
                        }
                        break;
                    case 1://跳过中间的
                        if (line.Contains("#region"))
                        {
                            region_cnt++;
                        }
                        else if (line.Contains("#endregion"))
                        {
                            region_cnt--;
                            if (region_cnt == 0)
                            {
                                //插入 自动代码
                                Gen_CS(sw);
                                
                                //下面都是尾
                                state = 2;
                            }
                        }
                        break;

                    case 2://记录尾巴
                        sw.WriteLine(line);
                        break;
                }
            }
            sr.Close();
            sw.Flush();
            sw.Close();

        }
        public void Gen_WS_XML()
        {
            PLCGroup plcgroup = new PLCGroup();
            plc_cnt = 1;
            for (int i=0;i<plc_cnt && i< mDeviceAddrs.EP.Count(); i++)
            {
                string ep_str = mDeviceAddrs.EP[i];
                plcgroup.Devices.Add(new PLCGroup.PLCDevice() { EP = Misc.StringConverter.ToIPEndPoint(ep_str) });
            }
            foreach (WS_ItemInfo ii in infos)
            {
                plcgroup.Variables.Add(new PLCGroup.PLCVariable()
                {
                    DeviceIndex = ii.plc_index,
                    Mode = ii.reg_type,
                    Addr = ii.mb_addr,
                    Type = ii.typeName,
                    Scale = ii.scale,
                    OwnerName = ii.objectName,
                    PropertyName = ii.propName
                });
            }
            string ws_path = Path.Combine(Path.GetDirectoryName(XlsxFile),"Generated","WS.xml");
            plcgroup.Save(ws_path);
        }


        protected void NotifyPropertyChanged(string propertyname) 
        {
            if (PropertyChanged != null)
            {
                PropertyChanged.Invoke(this, new PropertyChangedEventArgs(propertyname));
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
    }
}
