﻿using OfficeOpenXml;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;

namespace Autogen_WS
{
    public class DevMapperRule
    {
        /// <summary>
        /// PLC寄存器类型
        /// </summary>
        public string PLCVarType { get; set; }
        /// <summary>
        /// PLC寄存器首地址
        /// </summary>
        public int PLCAddr1st { get; set; }
        /// <summary>
        /// MB寄存器类型 [0区，1区，3区，4区]
        /// </summary>
        public string VarType { get; set; }
        /// <summary>
        /// MB寄存器首地址
        /// </summary>
        public int Addr1st { get; set; }
    }

    /// <summary>
    /// 设备转换表
    /// </summary>
    public class DevMapperRuleTable
    {
        public List<DevMapperRule> Tables = new List<DevMapperRule>();

        /// <summary>
        /// 读取 地址段映射
        /// </summary>
        /// <returns></returns>
        public bool Load(string xlsxfile)
        {
            Tables.Clear();

            //列序号 
            int i_plcVarType = 1;//PLC寄存器类型
            int i_plcAddr1st = 2;//PLC寄存器首地址
            int i_vartype = 3;//MB寄存器类型
            int i_addr1st = 4;//MB寄存器首地址



            //Regex r_variablename = new Regex(@"设备[0-9]_(读写|只读)(\w+)");
            Regex r_mode = new Regex(@"\[(0|1|3|4)区\]");

            using (ExcelPackage packages = new ExcelPackage(new FileInfo(xlsxfile)))
            {
                using (ExcelWorksheet sheet = packages.Workbook.Worksheets["地址段映射"])
                {
                    int i = 1;
                    while (true)
                    {
                        i++;
                        DevMapperRule cell = new DevMapperRule();
                        string str_plcvt;//PLC寄存器类型
                        string str_plca;//PLC寄存器首地址
                        string str_vt;//MB寄存器类型
                        string str_a;//MB寄存器首地址
                        try
                        {
                            str_plcvt = sheet.Cells[i, i_plcVarType].Value.ToString();
                            str_plca = sheet.Cells[i, i_plcAddr1st].Value.ToString();
                            str_vt = sheet.Cells[i, i_vartype].Value.ToString();
                            str_a = sheet.Cells[i, i_addr1st].Value.ToString();
                        }
                        catch
                        {
                            //完成，没有数据了
                            break;
                        }
                        if (string.IsNullOrEmpty(str_plcvt)) break;


                        int num;


                        cell.PLCVarType = str_plcvt;
                        if (!int.TryParse(str_plca, out num))
                            continue;
                        cell.PLCAddr1st = num;

                        Match m = r_mode.Match(str_vt);
                        if (!m.Success)
                            continue;

                        cell.VarType = m.Groups[1].Value;

                        if (!int.TryParse(str_a, out num))
                            continue;
                        cell.Addr1st = num - 1;

                        Tables.Add(cell);

                    }
                }
            }
            if (Tables.Count == 0)
                return false;
            return true;
        }
    }
}
