﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using OfficeOpenXml;
using System.ComponentModel;
using System.Text.RegularExpressions;
using System.Windows.Threading;

namespace Autogen_WS
{
    public class DataInfos : INotifyPropertyChanged
    {
        /// <summary>
        /// 输出的C#对象属性表
        /// </summary>
        List<WS_ItemInfo> infos = new List<WS_ItemInfo>();
        /// <summary>
        /// 设备EP
        /// </summary>
        DeviceAddr mDeviceAddrs = new DeviceAddr();
        /// <summary>
        /// 地址段映射
        /// </summary>
        DevMapperRuleTable mDevMapperRuleTable = new DevMapperRuleTable();

        /// <summary>
        /// 主界面的dispatcher
        /// </summary>
        public Action<string> ErrorMsgbox;


        #region 属性
        /// <summary>
        /// 进度
        /// </summary>
        public double Progress{get;set;}
        /// <summary>
        /// 输入的xlsx文件
        /// </summary>
        public string XlsxFile { get; set; }



        /// <summary>
        /// C#文件地址 WinderAccessory.cs
        /// </summary>
        public string CSharpPath_WinderAccessory
        {
            get;
            set;
        }

        /// <summary>
        /// C#文件地址 WinderInsideOutside.cs
        /// </summary>
        public string CSharpPath_WinderInsideOutside
        {
            get;
            set;
        }

        #endregion
        public DataInfos()
        {

        }


        public bool LoadXlsxFile()
        {
            //加载地址
            if (!mDeviceAddrs.Load(XlsxFile)) return false;
            //加载转换表
            if (!mDevMapperRuleTable.Load(XlsxFile)) return false;
            //加载 & 转换地址
            return LoadXlsxFile(XlsxFile);
        }

        /// <summary>
        /// 分析 "D(6120)" 分拆为寄存器类型, 寄存器地址
        /// </summary>
        /// <param name="plc_addr">寄存器地址字符串</param>
        /// <param name="type">寄存器类型</param>
        /// <param name="addr">寄存器地址</param>
        /// <returns>是否成功</returns>
        bool AnalysisAddr(string plc_addr, out string type, out int addr)
        {

            type = null;
            addr = 0;
            //string plc_addr = "设备1_读写4WUB3821";
            //Console.WriteLine("查找对象=" + plc_addr);


            Regex r = new Regex(@"(\w)\((\d+)\)");//分析 D(6120)
            Match m = r.Match(plc_addr);
            if (!m.Success)
                return false;
            string plc_vt = m.Groups[1].Value;
            int plc_a = int.Parse(m.Groups[2].Value);

            //查表
            var cells = from _cell in mDevMapperRuleTable.Tables where _cell.PLCVarType == plc_vt select _cell;
            if (cells.Count() == 0)
                return false;


            type = cells.First().VarType;
            //特别的， M0~...   D0~.... 是连续的
            //但 X0~X7 ,X10~X17, Y0~Y7 ,Y10~Y17, 不是连续的
            if (plc_vt == "X" || plc_vt == "Y")
            {
                plc_a = ((int)(plc_a / 10)) * 8 + plc_a % 10;
            }
            addr = (plc_a - cells.First().PLCAddr1st) + cells.First().Addr1st;

            return true;

        }


        /// <summary>
        /// 加载 HMI-地址表.xlsm
        /// </summary>
        /// <param name="xlsxfile"></param>
        /// <returns></returns>
        bool LoadXlsxFile(string xlsxfile)
        {
            infos.Clear();
            Progress = 0;

            if (!LoadXlsxFile_WinderAccessory(XlsxFile))
                return false;

            return LoadXlsxFile_WinderInsideOutSide(XlsxFile);

        }


        bool LoadXlsxFile_WinderAccessory(string xlsxfile)
        {
            int i_propDescription = 2;//描述
            int i_propName = 3;//属性名
            int i_typeName = 4;//类型
            int i_scale = 5;//放大
            int i_canWrite = 6;//可写
            int i_plc_addr = 7;//PLC寄存器




            using (ExcelPackage packages = new ExcelPackage(new FileInfo(xlsxfile)))
            {
                using (ExcelWorksheet sheet = packages.Workbook.Worksheets["参数"])
                {
                    // 生成C#对象属性列表

                    int iRow = 2;
                    while (true)
                    {
                        WS_ItemInfo ii = new WS_ItemInfo();

                        try
                        {
                            ii.propDescription = sheet.Cells[iRow, i_propDescription].Value.ToString();
                        }
                        catch
                        {
                            break;//没有描述，肯定就没有数据了
                        }


                        try
                        {
                            ii.propName = sheet.Cells[iRow, i_propName].Value.ToString();
                        }
                        catch
                        {
                            //没有名称异常
                            ErrorMsgbox?.Invoke("没有名称异常 描述=" + ii.propDescription);
                            return false;
                        }


                        try
                        {
                            ii.typeName = sheet.Cells[iRow, i_typeName].Value.ToString();
                        }
                        catch
                        {
                            //没有类型异常
                            ErrorMsgbox?.Invoke("没有类型异常 描述=" + ii.propDescription);
                            return false;
                        }

                        try
                        {
                            string scale_str = sheet.Cells[iRow, i_scale].Value.ToString();
                            if (!string.IsNullOrEmpty(scale_str))
                            {
                                double d;
                                if (double.TryParse(scale_str, out d))
                                {
                                    ii.scale = d;
                                }
                            }
                        }
                        catch
                        {

                        }

                        //把多余的空格删除
                        ii.propName = ii.propName.Trim();
                        ii.typeName = ii.typeName.Trim();
                        ii.propDescription = ii.propDescription.Trim();

                        ii.plc_index = 0;
                        ii.objectName = "Accessory";
                        //propertyname 不能有重复
                        if (infos.Any((info) => 
                        { return (info.propName == ii.propName) && (info.objectName == ii.objectName); }))
                        {
                            //propertyname 重复异常
                            ErrorMsgbox?.Invoke("propertyname 重复异常 描述=" + ii.propDescription);
                            return false;
                        }

                        string plc_addr;
                        try
                        {
                            plc_addr = sheet.Cells[iRow, i_plc_addr].Value.ToString();
                        }
                        catch
                        {
                            //空的
                            ErrorMsgbox?.Invoke("没有PLC地址 描述=" + ii.propDescription);
                            return false;
                        }

                        if (!AnalysisAddr(plc_addr, out ii.reg_type, out ii.mb_addr))
                        {
                            ErrorMsgbox?.Invoke("PLC地址格式出错 描述=" + ii.propDescription);
                            return false;
                        }

                        infos.Add(ii);

                        ii = ii.Clone();
                        
                    _next:
                        iRow++;
                        Progress += 1;
                        if (Progress >= 100)
                            Progress = 50;
                    }
                    Progress = 100;
                    return true;
                }
            }
        }

        bool LoadXlsxFile_WinderInsideOutSide(string xlsxfile)
        {
            int i_propDescription = 2;//描述
            int i_propName = 3;//属性名
            int i_typeName = 4;//类型
            int i_scale = 5;//放大
            int i_canWrite = 6;//可写
            int i_plc_addr = 7;//PLC寄存器, 内外两个


            using (ExcelPackage packages = new ExcelPackage(new FileInfo(xlsxfile)))
            {
                using (ExcelWorksheet sheet = packages.Workbook.Worksheets["收卷参数"])
                {
                    // 生成C#对象属性列表

                    int iRow = 2;
                    while (true)
                    {
                        WS_ItemInfo ii = new WS_ItemInfo();

                        try
                        {
                            ii.propDescription = sheet.Cells[iRow, i_propDescription].Value.ToString();
                        }
                        catch
                        {
                            break;//没有描述，肯定就没有数据了
                        }


                        try
                        {
                            ii.propName = sheet.Cells[iRow, i_propName].Value.ToString();
                        }
                        catch
                        {
                            //没有名称异常
                            ErrorMsgbox?.Invoke("没有名称异常 描述=" + ii.propDescription);
                            return false;
                        }



                        try
                        {
                            ii.typeName = sheet.Cells[iRow, i_typeName].Value.ToString();
                        }
                        catch
                        {
                            //没有类型异常
                            ErrorMsgbox?.Invoke("没有类型异常 描述=" + ii.propDescription);
                            return false;
                        }

                        try
                        {
                            string scale_str = sheet.Cells[iRow, i_scale].Value.ToString();
                            if (!string.IsNullOrEmpty(scale_str))
                            {
                                double d;
                                if (double.TryParse(scale_str, out d))
                                {
                                    ii.scale = d;
                                }
                            }
                        }
                        catch
                        {

                        }

                        //把多余的空格删除
                        ii.propName = ii.propName.Trim();
                        ii.typeName = ii.typeName.Trim();
                        ii.propDescription = ii.propDescription.Trim();

                        ii.plc_index = 0;

                        //内外收卷， 内=0，外=1
                        for (int i = 0; i < 2; i++)
                        {
                            ii.objectName = "Items[" + i.ToString()+"]";

                            if (infos.Any((info) =>
                            { return (info.propName == ii.propName) && (info.objectName == ii.objectName); }))
                            {
                                //propertyname 重复异常
                                ErrorMsgbox?.Invoke("propertyname 重复异常 objectName="+ii.objectName+" 描述=" + ii.propDescription);
                                return false;
                            }

                            string plc_addr;
                            try
                            {
                                plc_addr = sheet.Cells[iRow, i_plc_addr + i].Value.ToString();
                            }
                            catch
                            {
                                //空的
                                goto _next;
                            }

                            if (!AnalysisAddr(plc_addr, out ii.reg_type, out ii.mb_addr))
                            {
                                goto _next;
                            }

                            infos.Add(ii);

                            ii = ii.Clone();
                        }


                    _next:
                        iRow++;
                        Progress += 1;
                        if (Progress >= 100)
                            Progress = 50;
                    }
                    Progress = 100;
                    return true;
                }
            }
        }


        void Gen_CS_Core(StreamWriter sw, Dictionary<string, WS_ItemInfo> d)
        {
            sw.WriteLine("        #region 辅助代码生成");
            foreach (KeyValuePair<string, WS_ItemInfo> sii in d)
            {
                WS_ItemInfo info = sii.Value;
                sw.WriteLine("        /// <summary>");
                sw.WriteLine("        /// {0}", info.propDescription);
                sw.WriteLine("        /// </summary>");
                sw.Write("        public {0} {1}", info.GetPropertyType(), info.propName);
                sw.WriteLine(" { get; set; }");
                sw.WriteLine();
            }
            sw.WriteLine("        #endregion");
        }
        void Gen_CS_WinderAccessory(StreamWriter sw)
        {

            Dictionary<string, WS_ItemInfo> d = new Dictionary<string, WS_ItemInfo>();

            foreach (WS_ItemInfo ii in infos)
            {
                if (ii.objectName != "Accessory") continue;
                if (d.ContainsKey(ii.propName)) continue;
                d.Add(ii.propName, ii);
            }
            Gen_CS_Core(sw, d);

        }

        void Gen_CS_WinderInsideOutSide(StreamWriter sw)
        {

            Dictionary<string, WS_ItemInfo> d = new Dictionary<string, WS_ItemInfo>();

            foreach (WS_ItemInfo ii in infos)
            {
                if ((ii.objectName != "Items[0]") && (ii.objectName != "Items[1]")) continue;
                if (d.ContainsKey(ii.propName)) continue;
                d.Add(ii.propName, ii);
            }

            Gen_CS_Core(sw, d);
        }

        void Gen_CS(Action<StreamWriter> core, string objname, string csharppath)
        {
            //找到 #region 辅助代码生成 与最接近的 #endregion

            //StringBuilder sb = new StringBuilder();
            List<String> file_content_header = new List<string>();
            List<String> file_content_tailer = new List<string>();
            int state = 0;
            int region_cnt = 0;

            string weightc_new_path = Path.Combine(Path.GetDirectoryName(XlsxFile), "Generated", objname + ".cs");

            if (!Directory.Exists(Path.GetDirectoryName(weightc_new_path)))
                Directory.CreateDirectory(Path.GetDirectoryName(weightc_new_path));

            StreamWriter sw = new StreamWriter(weightc_new_path, false, Encoding.UTF8);

            if (File.Exists(csharppath))
            {
                StreamReader sr = new StreamReader(csharppath);

                //把整个文件读到 sb中
                while (!sr.EndOfStream)
                {
                    string line = sr.ReadLine();
                    switch (state)
                    {
                        case 0:
                            if (line.Contains("#region 辅助代码生成"))
                            {
                                //找到了
                                state = 1;
                                region_cnt = 1;
                            }
                            else
                            {
                                sw.WriteLine(line);
                            }
                            break;
                        case 1://跳过中间的
                            if (line.Contains("#region"))
                            {
                                region_cnt++;
                            }
                            else if (line.Contains("#endregion"))
                            {
                                region_cnt--;
                                if (region_cnt == 0)
                                {
                                    //插入 自动代码
                                    core(sw);

                                    //下面都是尾
                                    state = 2;
                                }
                            }
                            break;

                        case 2://记录尾巴
                            sw.WriteLine(line);
                            break;
                    }
                }

                sr.Close();
                sw.Flush();
                sw.Close();
            }
            else
            {
                sw.WriteLine("#region 辅助代码生成");
                core(sw);
                sw.WriteLine("#endregion");
                sw.Flush();
                sw.Close();
            }


        }

        public void Gen_CS()
        {
            Gen_CS(Gen_CS_WinderAccessory, "WinderAccessory", CSharpPath_WinderAccessory);
            Gen_CS(Gen_CS_WinderInsideOutSide, "WinderInsideOutside", CSharpPath_WinderInsideOutside);
        }

        public void Gen_WS_XML()
        {
            PLCGroup plcgroup = new PLCGroup();

            for (int i = 0; i < mDeviceAddrs.EP.Count(); i++)
            {
                string ep_str = mDeviceAddrs.EP[i];
                plcgroup.Devices.Add(new PLCGroup.PLCDevice() { EP = Misc.StringConverter.ToIPEndPoint(ep_str) });
            }
            foreach (WS_ItemInfo ii in infos)
            {
                plcgroup.Variables.Add(new PLCGroup.PLCVariable()
                {
                    DeviceIndex = ii.plc_index,
                    Mode = ii.reg_type,
                    Addr = ii.mb_addr,
                    Type = ii.typeName,
                    Scale = ii.scale,
                    OwnerName = ii.objectName,
                    PropertyName = ii.propName
                });
            }
            string ws_path = Path.Combine(Path.GetDirectoryName(XlsxFile), "Generated", "WS.xml");
            plcgroup.Save(ws_path);
        }


        public event PropertyChangedEventHandler PropertyChanged;
    }
}
