﻿using OfficeOpenXml;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;

namespace Autogen_WS
{
    public class DeviceTranslationCell
    {
        /// <summary>
        /// 变量名， 格式为 设备1_只读X0002
        /// </summary>
        public string PLCVarName { get; set; }

        /// <summary>
        /// plc序号
        /// </summary>
        public int PLCIndex { get; set; }
        /// <summary>
        /// 寄存器名称[1区，0区]  0,1,3,4
        /// </summary>
        public string MBVarType { get; set; }
        /// <summary>
        /// 地址
        /// </summary>
        public int MBAddr { get; set; }
    }
    public class DeviceTranslationTable
    {
        public List<DeviceTranslationCell> Tables = new List<DeviceTranslationCell>();




        /// <summary>
        /// 读取设备转换表
        /// </summary>
        /// <returns></returns>
        public bool Load(string xlsxfile,int plcCnt)
        {
            Tables.Clear();


            int i_variablename = 2;//变量名
            int i_mode = 6;//寄存器名称
            int i_addr = 8;//地址


            Regex r_mode = new Regex(@"\[(0|1|3|4)区\]");

            using (ExcelPackage packages = new ExcelPackage(new FileInfo(xlsxfile)))
            {
                for (int pidx = 0; pidx < plcCnt; pidx++)
                {
                    using (ExcelWorksheet sheet = packages.Workbook.Worksheets["设备"+pidx.ToString()])
                    {
                        int i = 1;
                        while (true)
                        {
                            i++;
                            DeviceTranslationCell cell = new DeviceTranslationCell();
                            cell.PLCIndex = pidx;

                            string str_vn;
                            string str_mode;
                            string str_addr;
                            try
                            {
                                str_vn = sheet.Cells[i, i_variablename].Value.ToString();
                                str_mode = sheet.Cells[i, i_mode].Value.ToString();
                                str_addr = sheet.Cells[i, i_addr].Value.ToString();
                            }
                            catch
                            {
                                //完成，没有数据了
                                break;
                            }
                            if (string.IsNullOrEmpty(str_vn)) break;
                            cell.PLCVarName = str_vn;

                            Match match = r_mode.Match(str_mode);
                            if (!match.Success)
                                continue;
                            cell.MBVarType = match.Groups[1].Value;
                            int num;
                            if (!int.TryParse(str_addr, out num))
                                continue;

                            
                            cell.MBAddr = num - 1;
                            Tables.Add(cell);

                        }
                    }
                }
            }
            if (Tables.Count == 0)
                return false;
            return true;
        }

        public static void Test()
        {
            string str_mode = "[3区]输入继电器";
            Regex r_mode = new Regex(@"\[(0|1|3|4)区\]");
            Match match = r_mode.Match(str_mode);

            if (!match.Success)
            {
                Console.WriteLine("match.Success=false");
            }
            else
            {
                Console.WriteLine(match.Groups[1].Value);
            }
        }
    }


}
